@@ -16,6 +16,8 @@
 #undef LT_REVISION
 #undef LT_AGE
 #undef USE_DMALLOC
+#undef USE_EFENCE
+#undef USE_LIBCALLOC
 
 #undef USE_GCRYPT
 #undef GNUTLS_VERSION
@@ -32,4 +34,6 @@
 #undef NO_TIME_T
 
 #undef DEFINE_SIZE_T
- 
\ No newline at end of file
+
+#undef ENABLE_SRP
+#undef ENABLE_ANON
