@@ -104,11 +104,11 @@ int _gnutls_find_dn(gnutls_datum * odn, gnutls_cert * cert)
 	int len, result;
 	int start, end;
 
-	if (asn1_create_structure
+	if ((result=asn1_create_structure
 	    (_gnutls_get_pkix(), "PKIX1Implicit88.Certificate", &dn,
-	     "dn") != ASN_OK) {
+	     "dn")) != ASN_OK) {
 		gnutls_assert();
-		return GNUTLS_E_ASN1_ERROR;
+		return result;
 	}
 
 	result = asn1_get_der(dn, cert->raw.data, cert->raw.size);
@@ -116,7 +116,7 @@ int _gnutls_find_dn(gnutls_datum * odn, gnutls_cert * cert)
 		/* couldn't decode DER */
 		gnutls_assert();
 		asn1_delete_structure(dn);
-		return GNUTLS_E_ASN1_PARSING_ERROR;
+		return result;
 	}
 
 	result = asn1_get_start_end_der(dn, cert->raw.data, cert->raw.size,
@@ -127,7 +127,7 @@ int _gnutls_find_dn(gnutls_datum * odn, gnutls_cert * cert)
 		/* couldn't decode DER */
 		gnutls_assert();
 		asn1_delete_structure(dn);
-		return GNUTLS_E_ASN1_PARSING_ERROR;
+		return result;
 	}
 	asn1_delete_structure(dn);
 
@@ -426,7 +426,7 @@ static int _gnutls_find_acceptable_client_cert(GNUTLS_STATE state,
 /* Generate client certificate
  */
 
-int _gnutls_gen_x509_client_certificate(GNUTLS_STATE state, opaque ** data)
+int _gnutls_gen_x509_certificate(GNUTLS_STATE state, opaque ** data)
 {
 	int ret, i;
 	opaque *pdata;
@@ -478,7 +478,7 @@ int _gnutls_gen_x509_client_certificate(GNUTLS_STATE state, opaque ** data)
 
 enum PGPKeyDescriptorType { PGP_KEY_FINGERPRINT, PGP_KEY };
 
-int _gnutls_gen_openpgp_client_certificate(GNUTLS_STATE state,
+int _gnutls_gen_openpgp_certificate(GNUTLS_STATE state,
 					   opaque ** data)
 {
 	int ret;
@@ -524,7 +524,7 @@ int _gnutls_gen_openpgp_client_certificate(GNUTLS_STATE state,
 	return ret;
 }
 
-int _gnutls_gen_openpgp_client_certificate_fpr(GNUTLS_STATE state,
+int _gnutls_gen_openpgp_certificate_fpr(GNUTLS_STATE state,
 					       opaque ** data)
 {
 	int ret, fpr_size, packet_size;
@@ -549,7 +549,7 @@ int _gnutls_gen_openpgp_client_certificate_fpr(GNUTLS_STATE state,
 	if (apr_cert_list_length > 0 && apr_cert_list[0].version == 4)
 		packet_size += 20 + 1;
 	else			/* empty certificate case */
-		return _gnutls_gen_openpgp_client_certificate(state, data);
+		return _gnutls_gen_openpgp_certificate(state, data);
 
 	(*data) = gnutls_malloc(packet_size);
 	pdata = (*data);
@@ -585,15 +585,15 @@ int _gnutls_gen_cert_client_certificate(GNUTLS_STATE state, opaque ** data)
 	case GNUTLS_CRT_OPENPGP:
 		if (_gnutls_openpgp_send_fingerprint(state) == 0)
 			return
-			    _gnutls_gen_openpgp_client_certificate(state,
+			    _gnutls_gen_openpgp_certificate(state,
 								   data);
 		else
 			return
-			    _gnutls_gen_openpgp_client_certificate_fpr
+			    _gnutls_gen_openpgp_certificate_fpr
 			    (state, data);
 
 	case GNUTLS_CRT_X509:
-		return _gnutls_gen_x509_client_certificate(state, data);
+		return _gnutls_gen_x509_certificate(state, data);
 
 	default:
 		gnutls_assert();
@@ -601,103 +601,13 @@ int _gnutls_gen_cert_client_certificate(GNUTLS_STATE state, opaque ** data)
 	}
 }
 
-
-
-/* Generate Server certificate
- */
-
-int _gnutls_gen_openpgp_server_certificate(GNUTLS_STATE state,
-					   opaque ** data)
-{
-	int ret;
-	opaque *pdata;
-	gnutls_cert *apr_cert_list;
-	gnutls_private_key *apr_pkey;
-	int apr_cert_list_length;
-
-	if ((ret =
-	     _gnutls_find_apr_cert(state, &apr_cert_list,
-				   &apr_cert_list_length,
-				   &apr_pkey)) < 0) {
-		gnutls_assert();
-		return ret;
-	}
-
-	ret = 3 + 1 + 3;
-	if (apr_cert_list_length > 0)
-		ret += apr_cert_list[0].raw.size;
-
-	(*data) = gnutls_malloc(ret);
-	pdata = (*data);
-
-	if (pdata == NULL) {
-		gnutls_assert();
-		return GNUTLS_E_MEMORY_ERROR;
-	}
-
-	WRITEuint24(ret - 3, pdata);
-	pdata += 3;
-
-	*pdata = 1;		/* whole key */
-	pdata++;
-
-	if (apr_cert_list_length > 0) {
-		WRITEdatum24(pdata, apr_cert_list[0].raw);
-		pdata += (3 + apr_cert_list[0].raw.size);
-	} else
-		WRITEuint24(0, pdata);
-
-	return ret;
-}
-
-int _gnutls_gen_x509_server_certificate(GNUTLS_STATE state, opaque ** data)
-{
-	int ret, i, pdatasize;
-	opaque *pdata;
-	gnutls_cert *apr_cert_list;
-	gnutls_private_key *apr_pkey;
-	int apr_cert_list_length;
-
-	if ((ret =
-	     _gnutls_find_apr_cert(state, &apr_cert_list,
-				   &apr_cert_list_length,
-				   &apr_pkey)) < 0) {
-		gnutls_assert();
-		return ret;
-	}
-
-	ret = 3;
-	for (i = 0; i < apr_cert_list_length; i++) {
-		ret += apr_cert_list[i].raw.size + 3;
-		/* hold size for uint24
-		 * and the certificate */
-	}
-
-	(*data) = gnutls_malloc(ret);
-	pdata = (*data);
-
-	if (pdata == NULL) {
-		gnutls_assert();
-		return GNUTLS_E_MEMORY_ERROR;
-	}
-	WRITEuint24(ret - 3, pdata);
-	pdata += 3;
-	for (i = 0; i < apr_cert_list_length; i++) {
-		WRITEdatum24(pdata, apr_cert_list[i].raw);
-		pdata += (3 + apr_cert_list[i].raw.size);
-	}
-	pdatasize = ret;
-
-	return pdatasize;
-}
-
 int _gnutls_gen_cert_server_certificate(GNUTLS_STATE state, opaque ** data)
 {
 	switch (state->security_parameters.cert_type) {
 	case GNUTLS_CRT_OPENPGP:
-		return _gnutls_gen_openpgp_server_certificate(state, data);
+		return _gnutls_gen_openpgp_certificate(state, data);
 	case GNUTLS_CRT_X509:
-		return _gnutls_gen_x509_server_certificate(state, data);
+		return _gnutls_gen_x509_certificate(state, data);
 	default:
 		gnutls_assert();
 		return GNUTLS_E_UNKNOWN_ERROR;
@@ -740,6 +650,12 @@ int _gnutls_proc_x509_server_certificate(GNUTLS_STATE state, opaque * data,
 
 	info = _gnutls_get_auth_info(state);
 
+	if (data == NULL || data_size == 0) {
+		gnutls_assert();
+		/* no certificate was sent */
+		return GNUTLS_E_NO_CERTIFICATE_FOUND;
+	}
+
 	DECR_LEN(dsize, 3);
 	size = READuint24(p);
 	p += 3;
@@ -749,32 +665,27 @@ int _gnutls_proc_x509_server_certificate(GNUTLS_STATE state, opaque * data,
 		/* no certificate was sent */
 		return GNUTLS_E_NO_CERTIFICATE_FOUND;
 	}
+
 	i = dsize;
-
-	DECR_LEN(dsize, 3);
-	len = READuint24(p);
-	p += 3;
-
-	for (; i > 0; len = READuint24(p), p += 3) {
+	while(i > 0) {
+		DECR_LEN(dsize, 3);
+		len = READuint24(p);
+		p += 3;
 		DECR_LEN(dsize, len);
 		peer_certificate_list_size++;
 		p += len;
 		i -= len + 3;
-		if (i > 0)
-			DECR_LEN(dsize, 3);
 	}
 
 	if (peer_certificate_list_size == 0) {
 		gnutls_assert();
-		return GNUTLS_E_UNEXPECTED_PACKET_LENGTH;
+		return GNUTLS_E_NO_CERTIFICATE_FOUND;
 	}
 
-	/* Now we start parsing the list (again).
-	 * We don't use DECR_LEN since the list has
-	 * been parsed before.
+	/* Ok we now allocate the memory to hold the
+	 * certificate list 
 	 */
-	dsize = data_size;
-	i = dsize;
+
 	peer_certificate_list =
 	    gnutls_calloc(1, sizeof(gnutls_cert) *
 			  (peer_certificate_list_size));
@@ -785,14 +696,15 @@ int _gnutls_proc_x509_server_certificate(GNUTLS_STATE state, opaque * data,
 	}
 
 	p = data + 3;
-	i = data_size - 3;
-	j = 0;
 
-	len = READuint24(p);
-	p += 3;
-	for (; i > 0; len = READuint24(p), p += 3) {
-		if (j >= peer_certificate_list_size)
-			break;
+	/* Now we start parsing the list (again).
+	 * We don't use DECR_LEN since the list has
+	 * been parsed before.
+	 */
+
+	for (j=0;j<peer_certificate_list_size;j++) {
+		len = READuint24(p);
+		p += 3;
 
 		tmp.size = len;
 		tmp.data = p;
@@ -807,8 +719,6 @@ int _gnutls_proc_x509_server_certificate(GNUTLS_STATE state, opaque * data,
 		}
 
 		p += len;
-		i -= len + 3;
-		j++;
 	}
 
 
@@ -871,6 +781,11 @@ int _gnutls_proc_openpgp_server_certificate(GNUTLS_STATE state,
 
 	info = _gnutls_get_auth_info(state);
 
+	if (data == NULL || data_size == 0) {
+		gnutls_assert();
+		return GNUTLS_E_NO_CERTIFICATE_FOUND;
+	}
+
 	DECR_LEN(dsize, 3);
 	size = READuint24(p);
 	p += 3;
