@@ -22,8 +22,7 @@
 void _gnutls_print_state(GNUTLS_STATE state);
 char * _gnutls_bin2hex(const unsigned char *old, const size_t oldlen);
 void _gnutls_dump_mpi(char* prefix,MPI a);
-char* _gnutls_packet2str( int packet);
-char* _gnutls_alert2str( int alert);
-char* _gnutls_handshake2str( int handshake);
+const char* _gnutls_packet2str( int packet);
+const char* _gnutls_handshake2str( int handshake);
 char* GET_CN( gnutls_datum);
 #endif
