@@ -27,20 +27,26 @@
 # include <string.h>
 # include <stdlib.h>
 # include <stdio.h>
+# include <ctype.h>
 #endif
 
-#ifdef HAVE_STDARG_H
-# include <stdarg.h>
+#ifdef NO_TIME_T
+ typedef unsigned int time_t;
+#endif
+
+#ifdef NO_SIZE_T
+ typedef unsigned int size_t;
+ typedef int ssize_t;
 #endif
 
 #ifdef HAVE_STRINGS_H
 # include <strings.h>
 #endif
 
-#include <time.h>
-#include <ctype.h>
+#ifdef HAVE_TIME_H
+# include <time.h>
+#endif
 
-/* for open */
 #ifdef HAVE_SYS_TYPES_H
 # include <sys/types.h>
 #endif
@@ -70,6 +76,9 @@
 /* only used native uint64 in 64 bit machines */
  typedef unsigned long int uint64;
 #else
+/* some systems had problems with long long int, thus,
+ * it is not used.
+ */
  typedef struct { unsigned char i[8]; } uint64;
 #endif
 
@@ -109,12 +118,4 @@
 # endif
 #endif
 
-#ifdef USE_VA_COPY
-# ifndef HAVE_VA_COPY
-# define VA_COPY __va_copy
-# else
-# define VA_COPY va_copy
-# endif
-#endif
-
 #endif /* defines_h */
