@@ -25,7 +25,7 @@
 extern "C" {
 #endif
 
-#define LIBGNUTLS_VERSION "0.3.91"
+#define LIBGNUTLS_VERSION "0.3.92"
 
 #include <sys/types.h>
 #include <time.h>
@@ -57,7 +57,6 @@ typedef enum GNUTLS_AlertDescription { GNUTLS_A_CLOSE_NOTIFY, GNUTLS_A_UNEXPECTE
 } GNUTLS_AlertDescription;
 
 typedef enum GNUTLS_CertificateStatus { 
-	GNUTLS_CERT_TRUSTED=1, 
 	GNUTLS_CERT_NOT_TRUSTED=2, 
 	GNUTLS_CERT_INVALID=4, 
 	GNUTLS_CERT_EXPIRED=8, GNUTLS_CERT_CORRUPTED=16,
@@ -101,6 +100,7 @@ typedef struct {
 int gnutls_init(GNUTLS_STATE * state, GNUTLS_ConnectionEnd con_end);
 void gnutls_deinit(GNUTLS_STATE state);
 int gnutls_bye( GNUTLS_STATE state, GNUTLS_CloseRequest how);
+#define gnutls_close gnutls_bye
 
 int gnutls_handshake( GNUTLS_STATE state);
 int gnutls_rehandshake( GNUTLS_STATE state);
@@ -132,6 +132,7 @@ int gnutls_error_is_fatal( int error);
 void gnutls_perror( int error);
 const char* gnutls_strerror( int error);
 
+void gnutls_record_set_cbc_protection(GNUTLS_STATE state, int prot);
 ssize_t gnutls_record_send( GNUTLS_STATE state, const void *data, size_t sizeofdata);
 ssize_t gnutls_record_recv( GNUTLS_STATE state, void *data, size_t sizeofdata);
 #define gnutls_read gnutls_record_recv
@@ -375,9 +376,12 @@ int gnutls_x509_verify_certificate( const gnutls_datum* cert_list, int cert_list
 
 /* Openpgp certificate stuff */
 int gnutls_openpgp_extract_key_name( const gnutls_datum *cert,
-                                                  gnutls_openpgp_name *name);
+                                 int idx,
+                                 gnutls_openpgp_name *dn );
+
 int gnutls_openpgp_extract_key_version( const gnutls_datum *cert );
-time_t gnutls_openpgp_extract_key_activation_time( const gnutls_datum *cert );
+
+time_t gnutls_openpgp_extract_key_creation_time( const gnutls_datum *cert );
 time_t gnutls_openpgp_extract_key_expiration_time( const gnutls_datum *cert );
 
 int gnutls_openpgp_verify_key( const gnutls_datum* keyring, 
@@ -440,8 +444,6 @@ int gnutls_b64_decode_fmt( const gnutls_datum *b64_data, char* result, int* resu
 #define GNUTLS_E_GOT_APPLICATION_DATA -38
 #define GNUTLS_E_RECORD_LIMIT_REACHED -39
 #define GNUTLS_E_ENCRYPTION_FAILED -40
-#define GNUTLS_E_ASN1_ERROR -41
-#define GNUTLS_E_ASN1_PARSING_ERROR -42 /* GNUTLS_A_BAD_CERTIFICATE */
 #define GNUTLS_E_X509_CERTIFICATE_ERROR -43
 #define GNUTLS_E_PK_ENCRYPTION_FAILED -44
 #define GNUTLS_E_PK_DECRYPTION_FAILED -45
@@ -458,13 +460,27 @@ int gnutls_b64_decode_fmt( const gnutls_datum *b64_data, char* result, int* resu
 #define GNUTLS_E_REQUESTED_DATA_NOT_AVAILABLE -56
 #define GNUTLS_E_PKCS1_WRONG_PAD -57
 #define GNUTLS_E_RECEIVED_ILLEGAL_EXTENSION -58
-#define GNUTLS_E_INTERNAL -59
+#define GNUTLS_E_INTERNAL_ERROR -59
 #define GNUTLS_E_CERTIFICATE_KEY_MISMATCH -60
 #define GNUTLS_E_UNSUPPORTED_CERTIFICATE_TYPE -61 /* GNUTLS_A_UNSUPPORTED_CERTIFICATE */
 #define GNUTLS_E_X509_UNKNOWN_SAN -62
 #define GNUTLS_E_DH_PRIME_UNACCEPTABLE -63
-#define GNUTLS_E_FILE -64
-#define GNUTLS_E_ASCII_ARMOR -65
+#define GNUTLS_E_FILE_ERROR -64
+#define GNUTLS_E_ASCII_ARMOR_ERROR -65
+#define GNUTLS_E_UNIX_TIME_LIMIT_EXCEEDED -66
+#define GNUTLS_E_ASN1_ELEMENT_NOT_FOUND -67
+#define GNUTLS_E_ASN1_IDENTIFIER_NOT_FOUND -68
+#define GNUTLS_E_ASN1_DER_ERROR -69
+#define GNUTLS_E_ASN1_VALUE_NOT_FOUND -70
+#define GNUTLS_E_ASN1_GENERIC_ERROR -71
+#define GNUTLS_E_ASN1_VALUE_NOT_VALID -72
+#define GNUTLS_E_ASN1_TAG_ERROR -73
+#define GNUTLS_E_ASN1_TAG_IMPLICIT -74
+#define GNUTLS_E_ASN1_TYPE_ANY_ERROR -75
+#define GNUTLS_E_ASN1_SYNTAX_ERROR -76
+#define GNUTLS_E_ASN1_DER_OVERFLOW -77
+#define GNUTLS_E_TOO_MANY_EMPTY_PACKETS -78
+#define GNUTLS_E_OPENPGP_UID_REVOKED -79
 
 #define GNUTLS_E_UNIMPLEMENTED_FEATURE -250
 
@@ -546,19 +562,19 @@ int gnutls_b64_decode_fmt( const gnutls_datum *b64_data, char* result, int* resu
 
 
 #define ASN_OK                    0
-#define ASN_FILE_NOT_FOUND        1
-#define ASN_ELEMENT_NOT_FOUND     2
-#define ASN_IDENTIFIER_NOT_FOUND  3
-#define ASN_DER_ERROR             4
-#define ASN_VALUE_NOT_FOUND       5
-#define ASN_GENERIC_ERROR         6
-#define ASN_VALUE_NOT_VALID       7
-#define ASN_TAG_ERROR             8
-#define ASN_TAG_IMPLICIT          9
-#define ASN_ERROR_TYPE_ANY       10
-#define ASN_SYNTAX_ERROR         11
-#define ASN_MEM_ERROR		 12
-#define ASN_DER_OVERFLOW         13
+#define ASN_FILE_NOT_FOUND        GNUTLS_E_FILE_ERROR
+#define ASN_ELEMENT_NOT_FOUND     GNUTLS_E_ASN1_ELEMENT_NOT_FOUND
+#define ASN_IDENTIFIER_NOT_FOUND  GNUTLS_E_ASN1_IDENTIFIER_NOT_FOUND
+#define ASN_DER_ERROR             GNUTLS_E_ASN1_DER_ERROR
+#define ASN_VALUE_NOT_FOUND       GNUTLS_E_ASN1_VALUE_NOT_FOUND
+#define ASN_GENERIC_ERROR         GNUTLS_E_ASN1_GENERIC_ERROR
+#define ASN_VALUE_NOT_VALID       GNUTLS_E_ASN1_VALUE_NOT_VALID
+#define ASN_TAG_ERROR             GNUTLS_E_ASN1_TAG_ERROR
+#define ASN_TAG_IMPLICIT          GNUTLS_E_ASN1_TAG_IMPLICIT
+#define ASN_ERROR_TYPE_ANY        GNUTLS_E_ASN1_TYPE_ANY_ERROR
+#define ASN_SYNTAX_ERROR          GNUTLS_E_ASN1_SYNTAX_ERROR
+#define ASN_MEM_ERROR		  GNUTLS_E_MEMORY_ERROR
+#define ASN_DER_OVERFLOW          GNUTLS_E_ASN1_DER_OVERFLOW
 
 
 /******************************************************/
