@@ -5,4 +5,4 @@ const void *_gnutls_get_kx_cred( GNUTLS_KEY key, KXAlgorithm algo, int *err);
 int _gnutls_generate_key(GNUTLS_KEY key);
 CredType gnutls_auth_get_type( GNUTLS_STATE state);
 void* _gnutls_get_auth_info( GNUTLS_STATE state);
-int _gnutls_auth_info_set( GNUTLS_STATE state, CredType type, int size);
+int _gnutls_auth_info_set( GNUTLS_STATE state, CredType type, int size, int allow_change);
