@@ -274,7 +274,7 @@ static ssize_t _gnutls_read( GNUTLS_STATE state, void *iptr, size_t sizeOfPtr, i
  */
 int _gnutls_io_clear_peeked_data( GNUTLS_STATE state) {
 char peekdata1[10];
-char *peekdata2;
+char *peekdata2 = NULL;
 char * peek;
 int ret, sum;
 
@@ -301,6 +301,9 @@ int ret, sum;
         	if (ret > 0) sum+=ret;
        	} while( ret==GNUTLS_E_INTERRUPTED || ret==GNUTLS_E_AGAIN || sum < RCVLOWAT);
 
+	/* This check is here to see if peekdata2 is malloced or
+	 * not. This is not clean I know.
+	 */
 	if (peek==peekdata2) {
 		gnutls_free(peekdata2);
 	}
@@ -511,6 +514,7 @@ static int _gnutls_buffer_get( gnutls_datum * buffer, const opaque ** ptr, size_
 	return 0;
 }
 
+
 /* This function is like write. But it does not return -1 on error.
  * It does return gnutls_errno instead.
  *
@@ -618,6 +622,35 @@ ssize_t _gnutls_io_write_buffered( GNUTLS_STATE state, const void *iptr, size_t
 
 }
 
+/* This is exactly like write_buffered, but will use two buffers to read
+ * from.
+ */
+ssize_t _gnutls_io_write_buffered2( GNUTLS_STATE state, const void *iptr, size_t n, const void* iptr2, size_t n2)
+{
+
+	if (n==0) {
+		return _gnutls_io_write_buffered( state, iptr2, n2);
+	} else {
+		opaque* sptr;
+		ssize_t ret;
+
+		sptr = gnutls_malloc( n+n2);
+		if (sptr==NULL) {
+			gnutls_assert();
+			return GNUTLS_E_MEMORY_ERROR;
+		}
+	
+		memcpy( sptr, iptr, n);
+		memcpy( &sptr[n], iptr2, n2);
+
+		ret = _gnutls_io_write_buffered( state, sptr, n+n2);
+		gnutls_free( sptr);
+	
+		return ret;
+	}
+}
+
+
 /* This function writes the data that are left in the
  * TLS write buffer (ie. because the previous write was
  * interrupted.
