@@ -33,7 +33,8 @@ int _gnutls_plaintext2TLSCompressed(GNUTLS_STATE state,
 	
 	data=NULL;
 	
-	size = gnutls_compress( state->security_parameters.write_compression_algorithm, plaintext.data, plaintext.size, &data);
+	size = gnutls_compress( state->connection_state.write_compression_state,
+		plaintext.data, plaintext.size, &data, MAX_RECORD_SIZE+1024);
 	if (size < 0) {
 		gnutls_assert();
 		return GNUTLS_E_COMPRESSION_FAILED;
@@ -54,7 +55,8 @@ int _gnutls_TLSCompressed2plaintext(GNUTLS_STATE state,
 
 	data=NULL;
 	
-	size = gnutls_decompress( state->security_parameters.read_compression_algorithm, compressed.data, compressed.size, &data);
+	size = gnutls_decompress( state->connection_state.read_compression_state,
+		compressed.data, compressed.size, &data, MAX_RECORD_SIZE);
 	if (size < 0) {
 		gnutls_assert();
 		return GNUTLS_E_DECOMPRESSION_FAILED;
