@@ -1,5 +1,5 @@
 /*
- *      Copyright (C) 2001 Nikos Mavroyanopoulos
+ *      Copyright (C) 2001,2002 Nikos Mavroyanopoulos
  *
  * This file is part of GNUTLS.
  *
@@ -25,6 +25,7 @@
 #include <gnutls_algorithms.h>
 #include <gnutls_num.h>
 #include <gnutls_datum.h>
+#include <gnutls_state.h>
 
 /* This function is to be called after handshake, when master_secret,
  *  client_random and server_random have been initialized. 
@@ -63,7 +64,7 @@ int _gnutls_set_keys(GNUTLS_STATE state, int hash_size, int IV_size, int key_siz
 			block_size, key_block);
 	} else { /* TLS 1.0 */
 		ret =
-		    gnutls_PRF( state->security_parameters.master_secret, TLS_MASTER_SIZE,
+		    _gnutls_PRF( state->security_parameters.master_secret, TLS_MASTER_SIZE,
 			       keyexp, strlen(keyexp), random, 2*TLS_RANDOM_SIZE, 
 			       block_size, key_block);
 	}
@@ -282,6 +283,10 @@ int rc;
 		gnutls_cipher_deinit(state->connection_state.
 				     read_cipher_state);
 
+	if (state->connection_state.read_compression_state != NULL)
+		gnutls_comp_deinit(state->connection_state.
+				     read_compression_state, 1);
+
 
 	mac_size =
 	    _gnutls_mac_get_digest_size(state->security_parameters.
@@ -316,7 +321,6 @@ int rc;
 
 		}
 
-
 		break;
 
 	case GNUTLS_CLIENT:
@@ -346,11 +350,21 @@ int rc;
 
 		break;
 
-	default:
+	default: /* this check is useless */
 		gnutls_assert();
 		return GNUTLS_E_UNKNOWN_ERROR;
 	}
 
+	state->connection_state.read_compression_state =
+	    gnutls_comp_init(state->security_parameters.
+			       read_compression_algorithm, 1);
+
+	if (state->connection_state.read_compression_state ==
+	    GNUTLS_COMP_FAILED) {
+		gnutls_assert();
+		return GNUTLS_E_UNKNOWN_COMPRESSION_ALGORITHM;
+	}
+
 	return 0;
 }
 
@@ -427,6 +441,10 @@ int rc;
 		gnutls_cipher_deinit(state->connection_state.
 				     write_cipher_state);
 
+	if (state->connection_state.write_compression_state != NULL)
+		gnutls_comp_deinit(state->connection_state.
+				     write_compression_state, 0);
+
 	mac_size =
 	    _gnutls_mac_get_digest_size(state->security_parameters.
 					write_mac_algorithm);
@@ -448,6 +466,7 @@ int rc;
 			return GNUTLS_E_UNKNOWN_CIPHER;
 		}
 
+
 		/* copy mac secrets from cipherspecs, to connection
 		 * state.
 		 */
@@ -491,6 +510,15 @@ int rc;
 		return GNUTLS_E_UNKNOWN_ERROR;
 	}
 
+
+	state->connection_state.write_compression_state =
+	    gnutls_comp_init(state->security_parameters.write_compression_algorithm, 0);
+
+	if (state->connection_state.write_compression_state == GNUTLS_COMP_FAILED) {
+		gnutls_assert();
+		return GNUTLS_E_UNKNOWN_COMPRESSION_ALGORITHM;
+	}
+
 	return 0;
 }
 
