@@ -52,7 +52,7 @@ static gnutls_error_entry error_algorithms[] = {
 	GNUTLS_ERROR_ENTRY( GNUTLS_E_DH_PRIME_UNACCEPTABLE, 1),
 	GNUTLS_ERROR_ENTRY( GNUTLS_E_UNEXPECTED_PACKET_LENGTH, 1),
 	GNUTLS_ERROR_ENTRY( GNUTLS_E_INVALID_SESSION, 1),
-	GNUTLS_ERROR_ENTRY( GNUTLS_E_INTERNAL, 1),
+	GNUTLS_ERROR_ENTRY( GNUTLS_E_INTERNAL_ERROR, 1),
 	GNUTLS_ERROR_ENTRY( GNUTLS_E_RECEIVED_ILLEGAL_EXTENSION, 1),
 	GNUTLS_ERROR_ENTRY( GNUTLS_E_UNABLE_SEND_DATA, 1),
 	GNUTLS_ERROR_ENTRY( GNUTLS_E_FATAL_ALERT_RECEIVED ,1),
@@ -85,8 +85,6 @@ static gnutls_error_entry error_algorithms[] = {
 	GNUTLS_ERROR_ENTRY( GNUTLS_E_PULL_ERROR, 1),
 	GNUTLS_ERROR_ENTRY( GNUTLS_E_PUSH_ERROR, 1),
 	GNUTLS_ERROR_ENTRY( GNUTLS_E_RECORD_LIMIT_REACHED, 1),
-	GNUTLS_ERROR_ENTRY( GNUTLS_E_ASN1_PARSING_ERROR, 1),
-	GNUTLS_ERROR_ENTRY( GNUTLS_E_ASN1_ERROR, 1),
 	GNUTLS_ERROR_ENTRY( GNUTLS_E_X509_CERTIFICATE_ERROR, 1),
 	GNUTLS_ERROR_ENTRY( GNUTLS_E_X509_UNKNOWN_SAN, 1),
 	GNUTLS_ERROR_ENTRY( GNUTLS_E_X509_UNSUPPORTED_CRITICAL_EXTENSION, 1),
@@ -100,8 +98,21 @@ static gnutls_error_entry error_algorithms[] = {
 	GNUTLS_ERROR_ENTRY( GNUTLS_E_INVALID_PARAMETERS, 1),
 	GNUTLS_ERROR_ENTRY( GNUTLS_E_INVALID_REQUEST, 1),
 	GNUTLS_ERROR_ENTRY( GNUTLS_E_ILLEGAL_PARAMETER, 1),
-	GNUTLS_ERROR_ENTRY( GNUTLS_E_FILE, 1),
-	GNUTLS_ERROR_ENTRY( GNUTLS_E_ASCII_ARMOR, 1),
+	GNUTLS_ERROR_ENTRY( GNUTLS_E_FILE_ERROR, 1),
+	GNUTLS_ERROR_ENTRY( GNUTLS_E_ASCII_ARMOR_ERROR, 1),
+	GNUTLS_ERROR_ENTRY( GNUTLS_E_UNIX_TIME_LIMIT_EXCEEDED, 1),
+	GNUTLS_ERROR_ENTRY( GNUTLS_E_ASN1_ELEMENT_NOT_FOUND, 1),
+	GNUTLS_ERROR_ENTRY( GNUTLS_E_ASN1_IDENTIFIER_NOT_FOUND, 1),
+	GNUTLS_ERROR_ENTRY( GNUTLS_E_ASN1_DER_ERROR, 1),
+	GNUTLS_ERROR_ENTRY( GNUTLS_E_ASN1_VALUE_NOT_FOUND, 1),
+	GNUTLS_ERROR_ENTRY( GNUTLS_E_ASN1_GENERIC_ERROR, 1),
+	GNUTLS_ERROR_ENTRY( GNUTLS_E_ASN1_VALUE_NOT_VALID, 1),
+	GNUTLS_ERROR_ENTRY( GNUTLS_E_ASN1_TAG_ERROR, 1),
+	GNUTLS_ERROR_ENTRY( GNUTLS_E_ASN1_TAG_IMPLICIT, 1),
+	GNUTLS_ERROR_ENTRY( GNUTLS_E_ASN1_TYPE_ANY_ERROR, 1),
+	GNUTLS_ERROR_ENTRY( GNUTLS_E_ASN1_SYNTAX_ERROR, 1),
+	GNUTLS_ERROR_ENTRY( GNUTLS_E_ASN1_DER_OVERFLOW, 1),
+	GNUTLS_ERROR_ENTRY( GNUTLS_E_TOO_MANY_EMPTY_PACKETS, 1),
 	{0}
 };
 
