@@ -40,6 +40,7 @@
 #include "gnutls_constate.h"
 #include <gnutls_record.h>
 #include <gnutls_alert.h>
+#include <gnutls_state.h>
 
 #ifdef HANDSHAKE_DEBUG
 #define ERR(x, y) _gnutls_handshake_log( "HSK: %s (%d)\n", x,y)
@@ -197,7 +198,7 @@ int _gnutls_finished(GNUTLS_STATE state, int type, void *ret)
 		mesg = CLIENT_MSG;
 	}
 
-	return gnutls_PRF(state->security_parameters.master_secret,
+	return _gnutls_PRF(state->security_parameters.master_secret,
 			  TLS_MASTER_SIZE, mesg, siz, concat, 36,
 			  12, ret);
 }
@@ -365,7 +366,7 @@ char * data;
 	if (state->gnutls_internals.handshake_mac_handle_sha==NULL ||
 		state->gnutls_internals.handshake_mac_handle_md5==NULL) {
 		gnutls_assert();
-		return GNUTLS_E_INTERNAL;
+		return GNUTLS_E_INTERNAL_ERROR;
 	}
 
 	/* We check if there are pending data to hash.
@@ -1817,8 +1818,8 @@ static int _gnutls_handshake_hash_init( GNUTLS_STATE state) {
   * GNUTLS_E_INTERRUPTED. In that case you may resume the handshake
   * (call this function again, until it returns ok)
   *
-  * If this function is called by a server after a rehandshake request
-  * then GNUTLS_E_GOT_APPLICATION_DATA or GNUTLS_E_WARNING_ALERT_RECEIVED 
+  * If this function is called by a server after a rehandshake request then
+  * GNUTLS_E_GOT_APPLICATION_DATA or GNUTLS_E_WARNING_ALERT_RECEIVED 
   * may be returned. Note that these are non fatal errors, only in the
   * case of a rehandshake. In that case they mean that the client
   * rejected the rehandshake request.
@@ -2330,13 +2331,13 @@ int _gnutls_remove_unwanted_ciphersuites(GNUTLS_STATE state,
 	 */
 
 	cert = NULL;
-
 	if (state->security_parameters.entity == GNUTLS_SERVER)
 		cert = _gnutls_server_find_cert(state, requested_pk_algo);
 
 	if (cert == NULL) {
 		/* No certificate was found 
 		 */
+		gnutls_assert();
 		alg_size = 0;
 		alg = NULL;
 	} else {
@@ -2372,9 +2373,9 @@ int _gnutls_remove_unwanted_ciphersuites(GNUTLS_STATE state,
 		/* if it is defined but had no credentials 
 		 */
 		if (_gnutls_get_kx_cred
-		    (state->gnutls_key, kx, NULL) == NULL)
+		    (state->gnutls_key, kx, NULL) == NULL) {
 			keep = 1;
-		else
+		} else
 		/* If there was no credentials to use with the specified
 		 * key exchange method, then just remove it.
 		 */
@@ -2384,7 +2385,7 @@ int _gnutls_remove_unwanted_ciphersuites(GNUTLS_STATE state,
 				if (state->security_parameters.entity ==
 				    GNUTLS_SERVER) {
 					/* here we check if the KX algorithm 
-					 * is compatible with the X.509 certificate.
+					 * is compatible with the certificate.
 					 */
 					for (j = 0; j < alg_size; j++) {
 						if (alg[j] == kx) {
