@@ -104,7 +104,6 @@ typedef enum crypt_algo { SRPSHA1_CRYPT, BLOWFISH_CRYPT=2 } crypt_algo;
 typedef enum ChangeCipherSpecType { GNUTLS_TYPE_CHANGE_CIPHER_SPEC=1 } ChangeCipherSpecType;
 
 typedef enum CertificateStatus { 
-	GNUTLS_CERT_TRUSTED=1, 
 	GNUTLS_CERT_NOT_TRUSTED=2, 
 	GNUTLS_CERT_INVALID=4, 
 	GNUTLS_CERT_EXPIRED=8, GNUTLS_CERT_CORRUPTED=16,
@@ -269,6 +268,7 @@ typedef struct GNUTLS_KEY_INT* GNUTLS_KEY;
 
 #include <gnutls_hash_int.h>
 #include <gnutls_cipher_int.h>
+#include <gnutls_compress_int.h>
 #include <gnutls_cert.h>
 
 typedef struct {
@@ -354,6 +354,8 @@ typedef struct {
 typedef struct {
 	GNUTLS_CIPHER_HANDLE write_cipher_state;
 	GNUTLS_CIPHER_HANDLE read_cipher_state;
+	GNUTLS_COMP_HANDLE   read_compression_state;
+	GNUTLS_COMP_HANDLE   write_compression_state;
 	gnutls_datum 	read_mac_secret;
 	gnutls_datum 	write_mac_secret;
 	uint64		read_sequence_number;
@@ -544,6 +546,8 @@ typedef struct {
 	 * record packet will have. */
 	GNUTLS_Version		default_record_version;
 
+	int			cbc_protection_hack;
+
 	/* If you add anything here, check _gnutls_handshake_internal_state_clear().
 	 */
 } GNUTLS_INTERNALS;
@@ -572,9 +576,6 @@ typedef struct {
 #define GNUTLS_DH_PARAMS _GNUTLS_DH_PARAMS*
 
 /* functions */
-int gnutls_PRF( opaque * secret, int secret_size, uint8 * label,
-		  int label_size, opaque * seed, int seed_size,
-		  int total_bytes, void* ret);
 void _gnutls_set_current_version(GNUTLS_STATE state, GNUTLS_Version version);
 GNUTLS_Version gnutls_protocol_get_version(GNUTLS_STATE state);
 void _gnutls_free_auth_info( GNUTLS_STATE state);
