@@ -38,6 +38,7 @@
 #include <netinet/in.h>
 #include <arpa/inet.h>
 #include <netdb.h>
+#include <assert.h>
 
 #define OPENPGP_NAME_SIZE GNUTLS_X509_CN_SIZE
 
@@ -86,35 +87,43 @@ file_exist(const char *file)
 
 typedef struct {
   int type;
+  int armored;
   size_t size;
   byte *data;
-} keyring_blob;
+} keybox_blob;
+
+typedef enum {
+  KBX_BLOB_FILE = 0x00,
+  KBX_BLOB_DATA = 0x01
+} keyring_blob_types;
 
 static int
-keyring_blob_new(keyring_blob **r_ctx)
+kbx_blob_new( keybox_blob **r_ctx )
 {
-  keyring_blob *c;
+  keybox_blob *c;
   
-  if (!r_ctx)
+  if ( !r_ctx )
     return GNUTLS_E_INVALID_PARAMETERS;
   
   c = cdk_alloc_clear( sizeof * c);
+  if ( !c )
+    return GNUTLS_E_MEMORY_ERROR;
   *r_ctx = c;
 
   return 0;
-}
+} /* kbx_blob_new */
 
 static void
-keyring_blob_release(keyring_blob *ctx)
+kbx_blob_release( keybox_blob *ctx )
 {
   if (!ctx)
     return;
   cdk_free(ctx->data);
   cdk_free(ctx);
-}
+} /* kbx_blob_release */
 
 static KEYDB_HD
-keyring_to_keydb(keyring_blob *blob)
+kbx_to_keydb( keybox_blob *blob )
 {
   KEYDB_HD khd = NULL;
 
@@ -122,64 +131,76 @@ keyring_to_keydb(keyring_blob *blob)
     return NULL;
   
   khd = cdk_alloc_clear(sizeof *khd);
+  if ( !khd )
+    return NULL;
   khd->used = 1;
-  if (blob->type == 0x00) /* file */
-      khd->name = cdk_strdup(blob->data);   
-  else if (blob->type == 0x01) /* data */
-    {
-      cdk_iobuf_new(&khd->buf, blob->size);
-      cdk_iobuf_write(khd->buf, blob->data, blob->size);
+  if ( blob->type == KBX_BLOB_FILE )
+    { /* file */
+      khd->name = cdk_strdup( blob->data );
+      khd->type = blob->armored? KEYDB_TYPE_ARMORED: KEYDB_TYPE_KEYRING;
+    }
+  else if ( blob->type == KBX_BLOB_DATA )
+    { /* data */
+      cdk_iobuf_new( &khd->buf, blob->size );
+      cdk_iobuf_write( khd->buf, blob->data, blob->size );
+      khd->type = KEYDB_TYPE_DATA;
     }
   else /* error */
     {
-      cdk_free(khd);
+      cdk_free( khd );
       khd = NULL;
     }
   
   return khd;
-}
-                     
-/* Extract a keyring blob from the given position. */
-static keyring_blob*
-read_keyring_blob(const gnutls_datum* keyring, size_t pos)
+} /* kbx_to_keydb */
+
+/* Extract a keybox blob from the given position. */
+static keybox_blob*
+kbx_read_blob( const gnutls_datum* keyring, size_t pos )
 {
-  keyring_blob *blob = NULL;
+  keybox_blob *blob = NULL;
   
-  if (!keyring || !keyring->data)
+  if ( !keyring || !keyring->data )
     return NULL;
 
   if (pos > keyring->size)
     return NULL;
     
-  keyring_blob_new(&blob);
+  kbx_blob_new( &blob );
   blob->type = keyring->data[pos];
-  if (blob->type < 0 || blob->type > 1)
+  if ( blob->type != KBX_BLOB_FILE &&
+       blob->type != KBX_BLOB_DATA )
     {
-      keyring_blob_release(blob);
+      kbx_blob_release( blob );
       return NULL;
     }
-  blob->size = buffer_to_u32( keyring->data+pos+1 );
+  blob->armored = keyring->data[pos+1];
+  blob->size = buffer_to_u32( keyring->data+pos+2 );
   if (!blob->size)
     {
-      keyring_blob_release(blob);
+      kbx_blob_release( blob );
       return NULL;
     }
   blob->data = cdk_alloc_clear(blob->size + 1);
-  memcpy(blob->data, keyring->data+(pos+5), blob->size);
+  if ( !blob->data )
+    return NULL;
+  memcpy(blob->data, keyring->data+(pos+6), blob->size);
   blob->data[blob->size] = '\0';
   
   return blob;
-}
+} /* kbx_read_blob */
 
 /* Creates a keyring blob from raw data
  *
  * Format:
  * 1 octet  type
+ * 1 octet  armored
  * 4 octet  size of blob
  * n octets data
  */
 static byte*
-conv_data_to_keyring(int type, const char *data, size_t size, size_t *r_size)
+kbx_data_to_keyring( int type, int enc, const char *data,
+                     size_t size, size_t *r_size )
 {
   byte *p = NULL;
 
@@ -187,17 +208,20 @@ conv_data_to_keyring(int type, const char *data, size_t size, size_t *r_size)
     return NULL;
   
   p = gnutls_malloc( 1+4+size );
-  p[0] = type; /* type: keyring name */
-  p[1] = (size >> 24) & 0xff;
-  p[2] = (size >> 16) & 0xff;
-  p[3] = (size >>  8) & 0xff;
-  p[4] = (size      ) & 0xff;
-  memcpy(p+5, data, size);
-  if (r_size)
-    *r_size = 1+4+size;
+  if ( !p )
+    return NULL;
+  p[0] = type; /* type: {keyring,name} */
+  p[1] = enc; /* encoded: {plain, armored} */
+  p[2] = (size >> 24) & 0xff;
+  p[3] = (size >> 16) & 0xff;
+  p[4] = (size >>  8) & 0xff;
+  p[5] = (size      ) & 0xff;
+  memcpy( p+6, data, size );
+  if ( r_size )
+    *r_size = 6+size;
   
-  return p; 
-}
+  return p;
+} /* kbx_data_to_keyring */
 
 static int
 is_file_armored(char *file)
@@ -218,6 +242,8 @@ is_file_armored(char *file)
           goto leave;
         }
       data = cdk_alloc_clear(f_stat.st_size+1);
+      if ( !data )
+        return 0;
       fread(data, 1, f_stat.st_size, fp);
       if ( strstr(data, "-----BEGIN PGP")
            && strstr(data, "-----END PGP") )
@@ -231,41 +257,27 @@ leave:
 }
 
 static int
-pkt_find_type(PACKET pkt, int type)
+kbnode_to_datum( KBNODE kb_pk, gnutls_datum *raw )
 {
-  struct packet_struct *p;
-  
-  if (!pkt)
-    return 0;
-  
-  for (p=pkt; p && p->pkttype; p=p->next)
-    {
-      if (p->pkttype == type)
-        return 1;
-    }
-  
-  return 0;
-}
-
-static int
-pkt_to_datum(PACKET pkt, gnutls_datum *raw)
-{
-  struct packet_struct *p;
-  byte *data;
+  KBNODE p = NULL;
+  byte *data = NULL;
   size_t n = 0;
   int rc = 0;
+  PACKET pkt = {0};
   IOBUF a;
   
-  if (!pkt || !raw)
+  if (!kb_pk || !raw)
     return GNUTLS_E_INVALID_PARAMETERS;
 
   /* fixme: conver the whole key */
-  for (p=pkt; p && p->pkttype; p=p->next)
+  for (p=kb_pk; p && p->pkt->pkttype; p=p->next)
     {
-      if (p->pkttype == PKT_PUBLIC_KEY)
+      if (p->pkt->pkttype == PKT_PUBLIC_KEY)
         {
           a = cdk_iobuf_temp();
-          rc = cdk_pkt_write_public_key(a, p->pkt.public_key);
+          pkt.pkttype = PKT_PUBLIC_KEY;
+          pkt.pkt.public_key = p->pkt->pkt.public_key;
+          rc = cdk_pkt_build( a, &pkt );
           if (rc)
             return GNUTLS_E_UNKNOWN_ERROR;
           data = cdk_iobuf_get_data(a, &n);
@@ -284,19 +296,21 @@ pkt_to_datum(PACKET pkt, gnutls_datum *raw)
 }
 
 static int
-datum_to_openpgp_pkt( const gnutls_datum *raw, PACKET *r_pkt )
+datum_to_kbnode( const gnutls_datum *raw, KBNODE *r_pkt )
 {
   IOBUF buf;
-  PACKET pkt = NULL;
+  KBNODE pkt = NULL;
+  int dummy = 0;
   int rc = 0;
 
   if (!raw || !r_pkt)
     return GNUTLS_E_INVALID_PARAMETERS;
 
-  cdk_iobuf_new(&buf, raw->size);
-  cdk_iobuf_write(buf, raw->data, raw->size);
-  rc = cdk_pkt_parse(buf, &pkt);
-  if ( rc != CDKERR_EOF )
+  cdk_iobuf_new( &buf, raw->size );
+  cdk_iobuf_write( buf, raw->data, raw->size );
+  rc = cdk_keydb_get_keyblock( buf, &pkt, &dummy );
+  /*rc = cdk_pkt_parse(buf, &pkt);*/
+  if ( rc && rc != CDKERR_EOF )
     {
       rc = GNUTLS_E_NO_CERTIFICATE_FOUND;
       goto leave;
@@ -305,7 +319,7 @@ datum_to_openpgp_pkt( const gnutls_datum *raw, PACKET *r_pkt )
     rc = 0;
 
 leave:
-  cdk_iobuf_close(buf);
+  cdk_iobuf_close( buf );
   *r_pkt = (!rc)? pkt : NULL;
 
   return rc;
@@ -335,57 +349,6 @@ iobuf_to_datum(IOBUF buf, gnutls_datum *raw)
   return rc;
 }
 
-static PKT_signature *
-openpgp_pkt_to_sig(PACKET pkt, size_t idx)
-{
-  struct packet_struct *p = NULL;
-  size_t n = 0;
-
-  for (p=pkt; p && p->pkttype; p=p->next)
-    {
-      if (p->pkttype == PKT_SIGNATURE && n == idx)
-        return p->pkt.signature;
-      else if (p->pkttype == PKT_SIGNATURE)
-        n++;
-    }
-  
-  return NULL;
-}
-
-static PKT_public_key *
-openpgp_pkt_to_pk(PACKET pkt, size_t idx)
-{
-  struct packet_struct *p = NULL;
-  size_t n = 0;
-
-  for (p=pkt; p && p->pkttype; p=p->next)
-    {
-      if (p->pkttype == PKT_PUBLIC_KEY && n == idx)
-        return p->pkt.public_key;
-      else if (p->pkttype == PKT_PUBLIC_KEY)
-        n++;
-    }
-  
-  return NULL;
-}
-
-static PKT_user_id *
-openpgp_pkt_to_uid(PACKET pkt, size_t idx)
-{
-  struct packet_struct *p = NULL;
-  size_t n = 0;
-
-  for (p=pkt; p && p->pkttype; p=p->next)
-    {
-      if (p->pkttype == PKT_USER_ID && n == idx)
-        return p->pkt.user_id;
-      else if (p->pkttype == PKT_USER_ID)
-        n++;
-    }
-  
-  return NULL;
-}
-
 static int
 openpgp_pk_to_gnutls_cert(gnutls_cert *cert, PKT_public_key *pk)
 {
@@ -405,15 +368,15 @@ openpgp_pk_to_gnutls_cert(gnutls_cert *cert, PKT_public_key *pk)
   cert->valid = 0; /* fixme: should set after the verification */
   cert->cert_type = GNUTLS_CRT_OPENPGP;
 
-  if (is_DSA(pk->pubkey_algo) || pk->pubkey_algo == PKE_RSA_S)
+  if (is_DSA(pk->pubkey_algo) || pk->pubkey_algo == GCRY_PK_RSA_S)
     cert->keyUsage = GNUTLS_X509KEY_DIGITAL_SIGNATURE;
-  else if (pk->pubkey_algo == PKE_RSA_E)
+  else if (pk->pubkey_algo == GCRY_PK_RSA_E)
     cert->keyUsage = GNUTLS_X509KEY_ENCIPHER_ONLY;
-  else if (pk->pubkey_algo == PKE_RSA_ES)
+  else if (pk->pubkey_algo == GCRY_PK_RSA)
     cert->keyUsage = GNUTLS_X509KEY_DIGITAL_SIGNATURE
                    | GNUTLS_X509KEY_ENCIPHER_ONLY;
 
-  cert->params_size = cdk_key_pk_get_nmpis(pk->pubkey_algo, 0);
+  cert->params_size = cdk_pk_get_npkey( pk->pubkey_algo );
   for (i=0; i<cert->params_size; i++)
     {      
       nbytes = pk->mpi[i].bytes+2;
@@ -443,13 +406,16 @@ openpgp_sig_to_gnutls_cert(gnutls_cert *cert, PKT_signature *sig)
   size_t nbytes = 0;
   size_t sigsize = 0;
   byte *data = NULL;
+  PACKET pkt = {0};
   
   if (!cert || !sig)
     return GNUTLS_E_INVALID_PARAMETERS;
 
   sigsize = 20 + sig->hashed_size + sig->unhashed_size + 2*MAX_MPI_BYTES;
   cdk_iobuf_new(&buf, sigsize);
-  rc = cdk_pkt_write_signature(buf, sig);
+  pkt.pkttype = PKT_SIGNATURE;
+  pkt.pkt.signature = sig;
+  rc = cdk_pkt_build( buf, &pkt );
   if (rc)
     goto leave;
   data = cdk_iobuf_get_data(buf, &nbytes);
@@ -481,48 +447,40 @@ leave:
  * GnuTLS specific data which is need to perform secret key operations.
  -*/
 int
-_gnutls_openpgp_key2gnutls_key(gnutls_private_key *pkey,
-                               gnutls_datum raw_key)
+_gnutls_openpgp_key2gnutls_key( gnutls_private_key *pkey,
+                                gnutls_datum raw_key )
 {
-  struct packet_struct *p = NULL;
+  KBNODE p = NULL, kb_sk;
   PKT_secret_key *sk = NULL;
-  PACKET pkt = NULL;
   IOBUF buf;
   int pke_algo, i, j, eof = 0;
-  int rc = 0;
   size_t nbytes = 0;
+  int rc = 0;
 
   if (!pkey || raw_key.size <= 0)
     return GNUTLS_E_INVALID_PARAMETERS;
 
-  cdk_secure_memory_init(); 
+  cdk_secmem_init( 16384 );
   cdk_iobuf_new(&buf, raw_key.size);
   cdk_iobuf_write(buf, raw_key.data, raw_key.size);
 
-  cdk_pkt_new(&pkt);
-  rc = cdk_keydb_enum_sk(buf, &pkt, &eof);
-  if ( (eof == 1 && !pkt) || rc)
+  rc = cdk_keydb_get_keyblock( buf, &kb_sk, &eof );
+  if ( !kb_sk || rc )
     {
       rc = GNUTLS_E_UNKNOWN_ERROR;
       goto leave;
     }
-  
-  for (p=pkt; p; p=p->next)
-    {
-      if (p->pkttype == PKT_SECRET_KEY)
-        {
-          sk = p->pkt.secret_key;
-          break;
-        }
-    }
-  if (sk == NULL)
+  p = cdk_kbnode_find( kb_sk, PKT_SECRET_KEY );
+  if ( p )
+    sk = p->pkt->pkt.secret_key;
+  if ( sk == NULL )
     {
       rc = GNUTLS_E_NO_CERTIFICATE_FOUND;
       goto leave;
     }
   
   pke_algo = sk->pk->pubkey_algo;
-  pkey->params_size = cdk_key_pk_get_nmpis(pke_algo, 0);
+  pkey->params_size = cdk_pk_get_npkey( pke_algo );
   for (i=0; i<pkey->params_size; i++)
     {
       nbytes = sk->pk->mpi[i].bytes+2;
@@ -535,8 +493,8 @@ _gnutls_openpgp_key2gnutls_key(gnutls_private_key *pkey,
           goto leave;
         }
     }
-  pkey->params_size += cdk_key_sk_get_nmpis(pke_algo);
-  for (j=0; j<cdk_key_sk_get_nmpis(pke_algo); j++, i++)
+  pkey->params_size += cdk_pk_get_nskey( pke_algo );
+  for (j=0; j<cdk_pk_get_nskey( pke_algo ); j++, i++)
     {
       nbytes = sk->mpi[j]->bytes+2;
       rc = gcry_mpi_scan(&pkey->params[i], GCRYMPI_FMT_PGP,
@@ -548,11 +506,11 @@ _gnutls_openpgp_key2gnutls_key(gnutls_private_key *pkey,
           goto leave;
         }
     }
-  if (is_ELG(pke_algo))
+  if ( is_ELG(pke_algo) )
     return GNUTLS_E_UNWANTED_ALGORITHM;
-  else if (is_DSA(pke_algo))
+  else if ( is_DSA(pke_algo) )
     pkey->pk_algorithm = GNUTLS_PK_DSA;
-  else if (is_RSA(pke_algo))
+  else if ( is_RSA(pke_algo) )
     pkey->pk_algorithm = GNUTLS_PK_RSA;
   rc = gnutls_set_datum(&pkey->raw, raw_key.data, raw_key.size);
   if (rc < 0)
@@ -563,7 +521,7 @@ _gnutls_openpgp_key2gnutls_key(gnutls_private_key *pkey,
 
 leave:
   cdk_iobuf_close(buf);
-  cdk_pkt_release(pkt);
+  cdk_kbnode_release( kb_sk );
 
   return rc;
 }
@@ -579,8 +537,7 @@ leave:
 int
 _gnutls_openpgp_cert2gnutls_cert(gnutls_cert *cert, gnutls_datum raw)
 {
-  struct packet_struct *p;
-  PACKET pkt = NULL;
+  KBNODE p, kb_pk = NULL;
   PKT_public_key *pk = NULL;
   int rc = 0;
   
@@ -588,17 +545,12 @@ _gnutls_openpgp_cert2gnutls_cert(gnutls_cert *cert, gnutls_datum raw)
     return GNUTLS_E_INVALID_PARAMETERS;
 
   memset(cert, 0, sizeof *cert);
-  rc = datum_to_openpgp_pkt(&raw, &pkt);
+  rc = datum_to_kbnode( &raw, &kb_pk );
   if (rc)
     return rc;
-  for (p=pkt; p && p->pkttype; p=p->next)
-    {
-      if (p->pkttype == PKT_PUBLIC_KEY)
-        {
-          pk = p->pkt.public_key;
-          break;
-        }
-    }
+  p = cdk_kbnode_find( kb_pk, PKT_PUBLIC_KEY );
+  if ( p )
+    pk = p->pkt->pkt.public_key;
   if (pk == NULL)
     {
       rc = GNUTLS_E_NO_CERTIFICATE_FOUND;
@@ -611,9 +563,11 @@ _gnutls_openpgp_cert2gnutls_cert(gnutls_cert *cert, gnutls_datum raw)
       rc = GNUTLS_E_MEMORY_ERROR;
       goto leave;
     }
-  rc = openpgp_pk_to_gnutls_cert(cert, p->pkt.public_key);
+  rc = openpgp_pk_to_gnutls_cert( cert, pk );
 
 leave:
+  cdk_kbnode_release( kb_pk );
+  
   return rc;
 }
 
@@ -632,18 +586,18 @@ gnutls_openpgp_get_key(gnutls_datum *key, const gnutls_datum *keyring,
                        key_attr_t by, opaque *pattern)
 {
   int rc = 0;
-  keyring_blob *blob = NULL;
+  keybox_blob *blob = NULL;
   KEYDB_HD khd = NULL;
-  PACKET pk = NULL;
+  KBNODE pk = NULL;
   KEYDB_SEARCH ks;
   
   if (!key || !keyring || by == KEY_ATTR_NONE)
     return GNUTLS_E_INVALID_PARAMETERS;
 
-  blob = read_keyring_blob(keyring, 0);
+  blob = kbx_read_blob( keyring, 0 );
   if (!blob)
     return GNUTLS_E_MEMORY_ERROR;
-  khd = keyring_to_keydb(blob);
+  khd = kbx_to_keydb( blob );
   ks.type = by;
   switch (by)
     {
@@ -664,25 +618,25 @@ gnutls_openpgp_get_key(gnutls_datum *key, const gnutls_datum *keyring,
       goto leave;
     }
   
-  rc = cdk_keydb_search_key(khd, &pk, &ks);
+  rc = cdk_keydb_search( khd, &ks, &pk );
   if (rc)
     {
-      rc = GNUTLS_E_NO_CERTIFICATE_FOUND; 
+      rc = GNUTLS_E_NO_CERTIFICATE_FOUND;
       goto leave;
     }    
 
-  if ( !pkt_find_type(pk, PKT_PUBLIC_KEY) )
+  if ( !cdk_kbnode_find( pk, PKT_PUBLIC_KEY ) )
     {
       rc = GNUTLS_E_NO_CERTIFICATE_FOUND;
       goto leave;
     }
 
-  rc = pkt_to_datum(pk, key);
+  rc = kbnode_to_datum( pk, key );
   
 leave:
-  cdk_free(khd);
-  cdk_pkt_release(pk);
-  keyring_blob_release(blob);
+  cdk_free( khd );
+  cdk_kbnode_release( pk );
+  kbx_blob_release( blob );
   
   return rc;
 }
@@ -692,16 +646,15 @@ gnutls_certificate_set_openpgp_key_mem(GNUTLS_CERTIFICATE_CREDENTIALS res,
                                        gnutls_datum *cert,
                                        gnutls_datum *key)
 {
-  struct packet_struct *p;
   gnutls_datum raw;
-  PACKET pk = NULL;
+  KBNODE kb_pk = NULL, pkt;
   int rc = 0;
   int i;
   
   if (!res || !key || !cert)
     return GNUTLS_E_INVALID_PARAMETERS;
 
-  rc = datum_to_openpgp_pkt(cert, &pk);
+  rc = datum_to_kbnode( cert, &kb_pk );
   if (rc)
     goto leave;
 
@@ -729,22 +682,24 @@ gnutls_certificate_set_openpgp_key_mem(GNUTLS_CERTIFICATE_CREDENTIALS res,
       return GNUTLS_E_MEMORY_ERROR; 
     }
 
-  for (i=1, p=pk; p && p->pkttype; p=p->next)
+  for (i=1, pkt=kb_pk; pkt && pkt->pkt->pkttype; pkt=pkt->next)
     {
       if (i > MAX_PARAMS_SIZE)
         break;
-      if (p->pkttype == PKT_PUBLIC_KEY)
+      if (pkt->pkt->pkttype == PKT_PUBLIC_KEY)
         {
           int n = res->ncerts;
+          PKT_public_key *pk = pkt->pkt->pkt.public_key;
           res->cert_list_length[n] = 1;
           gnutls_set_datum(&res->cert_list[n][0].raw, cert->data, cert->size);
-          openpgp_pk_to_gnutls_cert(&res->cert_list[n][0], p->pkt.public_key);
+          openpgp_pk_to_gnutls_cert( &res->cert_list[n][0], pk );
           i++;
         }
-      else if (p->pkttype == PKT_SIGNATURE)
+      else if (pkt->pkt->pkttype == PKT_SIGNATURE)
         {
           int n = res->ncerts;
-          openpgp_sig_to_gnutls_cert(&res->cert_list[n][0], p->pkt.signature); 
+          PKT_signature *sig = pkt->pkt->pkt.signature;
+          openpgp_sig_to_gnutls_cert( &res->cert_list[n][0], sig ); 
         }
     }
   
@@ -762,7 +717,7 @@ gnutls_certificate_set_openpgp_key_mem(GNUTLS_CERTIFICATE_CREDENTIALS res,
   gnutls_free_datum(&raw);
   
 leave:
-  cdk_pkt_release(pk);
+  cdk_kbnode_release( kb_pk );
   
   return rc;
 }
@@ -775,7 +730,7 @@ leave:
  *
  * This funtion is used to load OpenPGP keys into the GnuTLS structure.
  * It doesn't matter whether the keys are armored or but, but the files
- * should only contain one key.
+ * should only contain one key which should not be encrypted.
  **/
 int
 gnutls_certificate_set_openpgp_key_file(GNUTLS_CERTIFICATE_CREDENTIALS res,
@@ -783,9 +738,8 @@ gnutls_certificate_set_openpgp_key_file(GNUTLS_CERTIFICATE_CREDENTIALS res,
                                         char* KEYFILE)
 {
   IOBUF buf = NULL;
-  PACKET pkt = NULL;
+  KBNODE kb_pk = NULL, pkt;
   gnutls_datum raw;
-  struct packet_struct *p = NULL;
   armor_filter_s afx;
   int eof = 0, i;
   int rc = 0;
@@ -794,19 +748,19 @@ gnutls_certificate_set_openpgp_key_file(GNUTLS_CERTIFICATE_CREDENTIALS res,
     return GNUTLS_E_INVALID_PARAMETERS;
 
   if ( !file_exist(CERTFILE) || !file_exist(KEYFILE) )
-    return GNUTLS_E_FILE;
+    return GNUTLS_E_FILE_ERROR;
   
   if ( is_file_armored(CERTFILE) )
     {
       rc = cdk_iobuf_open(&buf, CERTFILE, IOBUF_MODE_RD);
       if (rc == -1)
-        return GNUTLS_E_FILE;
+        return GNUTLS_E_FILE_ERROR;
       memset(&afx, 0, sizeof afx);
       rc = cdk_iobuf_push_filter(buf, &afx, cdk_armor_filter);
       if (rc)
         {
           cdk_iobuf_close(buf);
-          rc = GNUTLS_E_ASCII_ARMOR;
+          rc = GNUTLS_E_ASCII_ARMOR_ERROR;
           goto leave;          
         }
       /*cdk_iobuf_close(buf);*/
@@ -815,10 +769,8 @@ gnutls_certificate_set_openpgp_key_file(GNUTLS_CERTIFICATE_CREDENTIALS res,
     {
       rc = cdk_iobuf_open(&buf, CERTFILE, IOBUF_MODE_RD);
       if (rc == -1)
-        return GNUTLS_E_FILE;
+        return GNUTLS_E_FILE_ERROR;
     }
-  cdk_pkt_new(&pkt);
-
   res->cert_list = gnutls_realloc(res->cert_list,
                                   (1+res->ncerts)*sizeof(gnutls_cert*));
   if (res->cert_list == NULL)
@@ -826,6 +778,7 @@ gnutls_certificate_set_openpgp_key_file(GNUTLS_CERTIFICATE_CREDENTIALS res,
       gnutls_assert();
       return GNUTLS_E_MEMORY_ERROR;
     }
+
   res->cert_list_length = gnutls_realloc(res->cert_list_length,
                                          (1+res->ncerts)*sizeof(int));
   if (res->cert_list_length == NULL)
@@ -842,25 +795,27 @@ gnutls_certificate_set_openpgp_key_file(GNUTLS_CERTIFICATE_CREDENTIALS res,
     }
 
   do {
-    rc = cdk_keydb_enum_pk(buf, &pkt, &eof);
-    if ( (eof == 1 && !pkt) || rc)
+    rc = cdk_keydb_get_keyblock( buf, &kb_pk, &eof );
+    if ( !kb_pk || rc )
       break;
-    for (i=1, p=pkt; p && p->pkttype; p=p->next)
+    for (i=1, pkt=kb_pk; pkt && pkt->pkt->pkttype; pkt=pkt->next)
       {
         if (i > MAX_PARAMS_SIZE)
           break;
-        if (p->pkttype == PKT_PUBLIC_KEY)
+        if (pkt->pkt->pkttype == PKT_PUBLIC_KEY)
           {
             int n = res->ncerts;
+            PKT_public_key *pk = pkt->pkt->pkt.public_key;
             res->cert_list_length[n] = 1;
             iobuf_to_datum(buf, &res->cert_list[n][0].raw);
-            openpgp_pk_to_gnutls_cert(&res->cert_list[n][0], p->pkt.public_key);
+            openpgp_pk_to_gnutls_cert( &res->cert_list[n][0], pk );
             i++;
           }
-        else if (p->pkttype == PKT_SIGNATURE)
+        else if (pkt->pkt->pkttype == PKT_SIGNATURE)
           {
             int n = res->ncerts;
-            openpgp_sig_to_gnutls_cert( &res->cert_list[n][0], p->pkt.signature);
+            PKT_signature *sig = pkt->pkt->pkt.signature;
+            openpgp_sig_to_gnutls_cert( &res->cert_list[n][0], sig );
           }
       }
   } while (!eof && !rc);
@@ -868,23 +823,23 @@ gnutls_certificate_set_openpgp_key_file(GNUTLS_CERTIFICATE_CREDENTIALS res,
   cdk_iobuf_close(buf);
   if (rc)
     {
-      cdk_pkt_release(pkt);
+      cdk_kbnode_release( kb_pk );
       rc = GNUTLS_E_UNKNOWN_ERROR;
       goto leave;
     }
-  cdk_pkt_release(pkt);
+  cdk_kbnode_release( kb_pk );
 
   if ( is_file_armored(KEYFILE) )
     {
       rc = cdk_iobuf_open(&buf, KEYFILE, IOBUF_MODE_RD);
       if (rc == -1)
-        return GNUTLS_E_FILE;
+        return GNUTLS_E_FILE_ERROR;
       memset(&afx, 0, sizeof afx);
       rc = cdk_iobuf_push_filter(buf, &afx, cdk_armor_filter);
       if (rc)
         {
           cdk_iobuf_close(buf);
-          rc = GNUTLS_E_ASCII_ARMOR;
+          rc = GNUTLS_E_ASCII_ARMOR_ERROR;
           goto leave;
         }
       /*cdk_iobuf_close(buf);*/
@@ -893,19 +848,20 @@ gnutls_certificate_set_openpgp_key_file(GNUTLS_CERTIFICATE_CREDENTIALS res,
     {
       rc = cdk_iobuf_open(&buf, KEYFILE, IOBUF_MODE_RD);
       if (rc == -1)
-        return GNUTLS_E_FILE;
+        return GNUTLS_E_FILE_ERROR;
     }
   iobuf_to_datum(buf, &raw);
   cdk_iobuf_close(buf);
   
-  res->ncerts++;
   res->pkey = gnutls_realloc(res->pkey,
-                             (res->ncerts)*sizeof(gnutls_private_key));
+                             (res->ncerts+1)*sizeof(gnutls_private_key));
   if (res->pkey == NULL)
     {
       gnutls_assert();
       return GNUTLS_E_MEMORY_ERROR;
     }
+
+  res->ncerts++;
   
   /* ncerts has been incremented before */
   rc =_gnutls_openpgp_key2gnutls_key( &res->pkey[res->ncerts-1], raw);
@@ -915,6 +871,25 @@ leave:
   return rc;
 }
 
+int
+gnutls_openpgp_count_key_names( const gnutls_datum *cert )
+{
+  KBNODE kb_pk = NULL, pkt;
+  int nuids = 0;
+  
+  if ( !cert )
+    return 0;
+
+  if ( datum_to_kbnode( cert, &kb_pk ) )
+    return 0;
+  for ( pkt=kb_pk; pkt; pkt=pkt->next )
+    {
+      if ( pkt->pkt->pkttype == PKT_USER_ID )
+        nuids++;
+    }
+  return nuids;
+} /* gnutls_openpgp_count_key_names */
+
 /**
  * gnutls_openpgp_extract_key_name - Extracts the userID
  * @cert: the raw data that contains the OpenPGP public key.
@@ -924,52 +899,71 @@ leave:
  **/
 int
 gnutls_openpgp_extract_key_name( const gnutls_datum *cert,
+                                 int idx,
                                  gnutls_openpgp_name *dn )
 {
-  PACKET pkt = NULL;
+  KBNODE kb_pk = NULL, pkt;
   PKT_user_id *uid = NULL;
-  char *p;
+  char *email;
   int rc = 0;
   int pos1 = 0, pos2 = 0;
   size_t size = 0;
+  int pos = 0;
 
   if (!cert || !dn)
     return GNUTLS_E_INVALID_PARAMETERS;
 
-  rc = datum_to_openpgp_pkt(cert, &pkt);
-  if (rc)
+  if ( idx < 0 || idx > gnutls_openpgp_count_key_names( cert ) )
+    return GNUTLS_E_UNKNOWN_ERROR;
+
+  memset(dn, 0, sizeof *dn);
+  rc = datum_to_kbnode( cert, &kb_pk );
+  if ( rc )
     return rc;
-  uid = openpgp_pkt_to_uid(pkt, 0);
-  if (!uid)
+  if ( !idx )
+    pkt = cdk_kbnode_find( kb_pk, PKT_USER_ID );
+  else
+    {
+      for ( pos=0, pkt=kb_pk; pkt; pkt=pkt->next )
+        {
+          if ( pkt->pkt->pkttype == PKT_USER_ID && ++pos == idx )
+            break;
+        }
+    }
+  if ( pkt )
+    uid = pkt->pkt->pkt.user_id;
+  if ( !uid )
     {
       rc = GNUTLS_E_UNKNOWN_ERROR;
       goto leave;
     }
-  memset(dn, 0, sizeof *dn);
-  size = uid->size < OPENPGP_NAME_SIZE? uid->size : OPENPGP_NAME_SIZE;
+  size = uid->len < OPENPGP_NAME_SIZE? uid->len : OPENPGP_NAME_SIZE-1;
   memcpy(dn->name, uid->name, size);
-  dn->name[size-1] = '\0'; /* make sure it's a string */
+  dn->name[size] = '\0'; /* make sure it's a string */
 
-  /*
-   * Extract the email address from the userID string and save it to
-   * the email field.
-   */
-  p = strchr(uid->name, '<');
-  if (p)
-    pos1 = p-uid->name+1;
-  p = strchr(uid->name, '>');
-  if (p)
-    pos2 = p-uid->name+1;
+  /* Extract the email address from the userID string and save
+     it to the email field. */
+  email = strchr(uid->name, '<');
+  if ( email )
+    pos1 = email-uid->name+1;
+  email = strchr(uid->name, '>');
+  if ( email )
+    pos2 = email-uid->name+1;
   if (pos1 && pos2)
     {
       pos2 -= pos1;
-      size = pos2 < OPENPGP_NAME_SIZE? pos2 : OPENPGP_NAME_SIZE;
+      size = pos2 < OPENPGP_NAME_SIZE? pos2 : OPENPGP_NAME_SIZE-1;
       memcpy(dn->email, uid->name+pos1, size);
       dn->email[size-1] = '\0'; /* make sure it's a string */
     }
+  if ( uid->is_revoked )
+    {
+      rc = GNUTLS_E_OPENPGP_UID_REVOKED;
+      goto leave; 
+    }
   
 leave:
-  cdk_pkt_release(pkt);
+  cdk_kbnode_release( kb_pk );
   
   return rc;
 }
@@ -983,19 +977,18 @@ leave:
 int
 gnutls_openpgp_extract_key_version( const gnutls_datum *cert )
 {
-  PACKET pkt = NULL;
-  PKT_public_key *pk = NULL;
+  KBNODE kb_pk = NULL, pkt;
   int version = 0;
 
   if (!cert)
     return GNUTLS_E_INVALID_PARAMETERS;
 
-  if ( datum_to_openpgp_pkt(cert, &pkt) )
+  if ( datum_to_kbnode( cert, &kb_pk ) )
     return 0;
-  pk = openpgp_pkt_to_pk(pkt, 0);
-  if (pk)
-    version = pk->version;
-  cdk_pkt_release(pkt);
+  pkt = cdk_kbnode_find( kb_pk, PKT_PUBLIC_KEY );
+  if ( pkt )
+    version = pkt->pkt->pkt.public_key->version;
+  cdk_kbnode_release( kb_pk );
 
   return version;
 }
@@ -1009,19 +1002,18 @@ gnutls_openpgp_extract_key_version( const gnutls_datum *cert )
 time_t
 gnutls_openpgp_extract_key_creation_time( const gnutls_datum *cert )
 {
-  PACKET pkt = NULL;
-  PKT_public_key *pk = NULL;
+  KBNODE kb_pk = NULL, pkt;
   time_t timestamp = 0;
 
   if (!cert)
     return GNUTLS_E_INVALID_PARAMETERS;
   
-  if ( datum_to_openpgp_pkt(cert, &pkt) )
+  if ( datum_to_kbnode( cert, &kb_pk ) )
     return 0;
-  pk = openpgp_pkt_to_pk(pkt, 0);
-  if (pk)
-    timestamp = pk->timestamp;
-  cdk_pkt_release(pkt);
+  pkt = cdk_kbnode_find( kb_pk, PKT_PUBLIC_KEY );
+  if ( pkt )
+    timestamp = pkt->pkt->pkt.public_key->timestamp;
+  cdk_kbnode_release( kb_pk );
   
   return timestamp;
 }
@@ -1036,19 +1028,18 @@ gnutls_openpgp_extract_key_creation_time( const gnutls_datum *cert )
 time_t
 gnutls_openpgp_extract_key_expiration_time( const gnutls_datum *cert )
 {
-  PACKET pkt = NULL;
-  PKT_public_key *pk = NULL;
+  KBNODE kb_pk = NULL, pkt;
   time_t expiredate = 0;
 
   if (!cert)
     return GNUTLS_E_INVALID_PARAMETERS;
   
-  if ( datum_to_openpgp_pkt(cert, &pkt) )
+  if ( datum_to_kbnode( cert, &kb_pk ) )
     return 0;
-  pk = openpgp_pkt_to_pk(pkt, 0);
-  if (pk)
-    expiredate = pk->expiredate;
-  cdk_pkt_release(pkt);
+  pkt = cdk_kbnode_find( kb_pk, PKT_PUBLIC_KEY );
+  if ( pkt )
+    expiredate = pkt->pkt->pkt.public_key->expiredate;
+  cdk_kbnode_release( kb_pk );
   
   return expiredate;
 }
@@ -1061,7 +1052,7 @@ _gnutls_openpgp_get_key_trust(const char *trustdb,
   int flags = 0;
   int ot = 0, trustval = 0;
   int rc = 0;
-  PACKET pkt;
+  KBNODE kb_pk = NULL, pkt;
   PKT_public_key *pk = NULL;
   IOBUF buf;
 
@@ -1069,12 +1060,14 @@ _gnutls_openpgp_get_key_trust(const char *trustdb,
     return GNUTLS_E_NO_CERTIFICATE_FOUND;
 
   *r_success = 0;
-  rc = datum_to_openpgp_pkt(key, &pkt);
-  if (rc)
+  rc = datum_to_kbnode( key, &kb_pk );
+  if ( rc )
     return GNUTLS_E_NO_CERTIFICATE_FOUND;
-  
-  pk = openpgp_pkt_to_pk(pkt, 0);
-  if (!pk)
+
+  pkt = cdk_kbnode_find( kb_pk, PKT_PUBLIC_KEY );
+  if ( pkt )
+    pk = pkt->pkt->pkt.public_key;
+  if ( !pk )
     return GNUTLS_E_NO_CERTIFICATE_FOUND;
 
   rc = cdk_iobuf_open( &buf, trustdb, IOBUF_MODE_RD );
@@ -1083,11 +1076,12 @@ _gnutls_openpgp_get_key_trust(const char *trustdb,
       trustval = GNUTLS_E_NO_CERTIFICATE_FOUND;
       goto leave;
     }
-  rc = cdk_trustdb_find_ownertrust(buf, pk, &ot, &flags);
-  cdk_iobuf_close(buf);
-  if (rc)
-    {
-      rc = GNUTLS_E_NO_CERTIFICATE_FOUND;
+  rc = cdk_trustdb_get_ownertrust( buf, pk, &ot, &flags );
+  cdk_iobuf_close( buf );
+  if ( rc ) /* no ownertrust record was found */
+    {      
+      trustval = 0;
+      *r_success = 1;
       goto leave;
     }
 
@@ -1112,15 +1106,16 @@ _gnutls_openpgp_get_key_trust(const char *trustdb,
       
     case TRUST_FULLY:
     case TRUST_ULTIMATE:
-      trustval |= GNUTLS_CERT_TRUSTED;
+      trustval |= 1; /* means okay */
       *r_success = 1;
       break;
     }      
 
 leave:
+  cdk_kbnode_release( kb_pk );
   return trustval;
 }
-               
+
 /**
  * gnutls_openpgp_verify_key - Verify all signatures on the key
  * @cert_list: the structure that holds the certificates.
@@ -1136,9 +1131,9 @@ gnutls_openpgp_verify_key( const char *trustdb,
                            const gnutls_datum* cert_list,
                            int cert_list_length )
 {
-  PACKET pkt = NULL;
+  KBNODE kb_pk = NULL;
   KEYDB_HD khd = NULL;
-  keyring_blob *blob = NULL;
+  keybox_blob *blob = NULL;
   int rc = 0;
   int status = 0;
   
@@ -1148,10 +1143,10 @@ gnutls_openpgp_verify_key( const char *trustdb,
   if (cert_list_length != 1 || !keyring->size)
     return GNUTLS_CERT_INVALID;
 
-  blob = read_keyring_blob(keyring, 0);
+  blob = kbx_read_blob(keyring, 0);
   if (!blob)
     return GNUTLS_CERT_INVALID;
-  khd = keyring_to_keydb(blob);
+  khd = kbx_to_keydb(blob);
   if (!khd)
     {
       rc = GNUTLS_CERT_INVALID;
@@ -1165,33 +1160,15 @@ gnutls_openpgp_verify_key( const char *trustdb,
       if (!success)
         goto leave;
     }
-  
-  rc = datum_to_openpgp_pkt(cert_list, &pkt);
+
+  rc = datum_to_kbnode( cert_list, &kb_pk );
   if (rc)
     {
       goto leave;
       return GNUTLS_CERT_INVALID;
     }
-  
-  {
-    struct packet_struct *p;
-    PKT_user_id *id = NULL;
-    PKT_signature *sig = NULL;
-    for (p=pkt; p && p->pkttype; p=p->next)
-      {
-        if (p->pkttype == PKT_SIGNATURE)
-          sig = p->pkt.signature;
-        if (p->pkttype == PKT_USER_ID && sig && !sig->userid_hash)
-          {
-              id = p->pkt.user_id;
-            if (sig->sig_class >= 0x10 && sig->sig_class <= 0x13)
-              sig->userid_hash  = cdk_userid_create_hash(id);
-            id = NULL;
-          }
-      }
-  }
-  
-  rc = cdk_key_check_sigs(pkt, khd, &status);
+
+  rc = cdk_key_check_sigs( kb_pk, khd, &status );
   if (rc == CDKERR_NOKEY)
     rc = 0; /* fixme */
       
@@ -1211,8 +1188,10 @@ gnutls_openpgp_verify_key( const char *trustdb,
     }
 
 leave:
-  keyring_blob_release(blob);
-  cdk_free(khd);
+  kbx_blob_release( blob );
+  cdk_free( khd );
+  cdk_kbnode_release( kb_pk );
+  
   return rc;
 }
 
@@ -1228,32 +1207,29 @@ leave:
 int
 gnutls_openpgp_fingerprint(const gnutls_datum *cert, byte *fpr, size_t *fprlen)
 {
-  PACKET pkt = NULL;
+  KBNODE kb_pk = NULL, pkt;
   PKT_public_key *pk = NULL;
-  struct packet_struct *p;
   int rc = 0;
   
   if (!cert || !fpr || !fprlen)
     return GNUTLS_E_UNKNOWN_ERROR;
 
   *fprlen = 0;
-  rc = datum_to_openpgp_pkt(cert, &pkt);
+  rc = datum_to_kbnode( cert, &kb_pk );
   if (rc)
     return rc;
-  for (p=pkt; p && p->pkttype; p=p->next)
-    {
-      if (p->pkttype == PKT_PUBLIC_KEY)
-        {
-          pk = p->pkt.public_key;
-          if ( is_RSA(pk->pubkey_algo) && pk->version < 4 )
-            *fprlen = 16;
-          else
-            *fprlen = 20;
-          cdk_key_create_fpr(pk, fpr);
-        }
-    }
+  pkt = cdk_kbnode_find( kb_pk, PKT_PUBLIC_KEY );
+  if ( pkt )
+    pk = pkt->pkt->pkt.public_key;
+  if ( !pk )
+    return GNUTLS_E_UNKNOWN_ERROR;
+
+  *fprlen = 20;
+  if ( is_RSA(pk->pubkey_algo) && pk->version < 4 )
+    *fprlen = 16;
+  cdk_pk_get_fingerprint( pk, fpr );
   
-  return 0;  
+  return 0;
 }
 
 /**
@@ -1266,26 +1242,24 @@ gnutls_openpgp_fingerprint(const gnutls_datum *cert, byte *fpr, size_t *fprlen)
 int
 gnutls_openpgp_keyid( const gnutls_datum *cert, uint32 *keyid )
 {
-  PACKET pkt;
+  KBNODE kb_pk = NULL, pkt;
   PKT_public_key *pk = NULL;
-  struct packet_struct *p;
   int rc = 0;
   
   if (!cert || !keyid)
     return GNUTLS_E_UNKNOWN_ERROR;
 
-  rc = datum_to_openpgp_pkt(cert, &pkt);
+  rc = datum_to_kbnode( cert, &kb_pk );
   if (rc)
     return rc;
-  for (p=pkt; p && p->pkttype; p=p->next)
-    {
-      if (p->pkttype == PKT_PUBLIC_KEY)
-        {
-          pk = p->pkt.public_key;
-          cdk_key_keyid_from_pk(pk, (u32*)keyid);
-        }
-    }
 
+  pkt = cdk_kbnode_find( kb_pk, PKT_PUBLIC_KEY );
+  if ( pkt )
+    pk = pkt->pkt->pkt.public_key;
+  if ( !pk )
+    return GNUTLS_E_UNKNOWN_ERROR;
+  cdk_pk_get_keyid( pk, (u32 *)keyid );
+  
   return 0;
 }
 
@@ -1303,11 +1277,13 @@ gnutls_openpgp_add_keyring_file(gnutls_datum *keyring, const char *name)
 {
   byte *blob;
   size_t nbytes;
+  int enc = 0;
   
   if (!keyring || !name)
     return GNUTLS_E_INVALID_PARAMETERS;
 
-  blob = conv_data_to_keyring(0x00, name, strlen(name), &nbytes);
+  enc = is_file_armored( (char*)name );
+  blob = kbx_data_to_keyring( KBX_BLOB_FILE, enc, name, strlen(name), &nbytes);
   if (blob && nbytes)
     { 
       if ( gnutls_datum_append( keyring, blob, nbytes ) < 0 )
@@ -1340,8 +1316,8 @@ gnutls_openpgp_add_keyring_mem(gnutls_datum *keyring,
   if (!keyring || !data || !len)
     return GNUTLS_E_INVALID_PARAMETERS;
   
-  blob = conv_data_to_keyring(0x01, data, len, &nbytes);
-  if (blob && nbytes)
+  blob = kbx_data_to_keyring( KBX_BLOB_DATA, 0, data, len, &nbytes );
+  if ( blob && nbytes )
     {
       if ( gnutls_datum_append( keyring, blob, nbytes ) < 0 )
         {
@@ -1362,7 +1338,7 @@ gnutls_certificate_set_openpgp_keyring_file(GNUTLS_CERTIFICATE_CREDENTIALS c,
     return GNUTLS_E_INVALID_PARAMETERS;
 
   if ( !file_exist(file) )
-    return GNUTLS_E_FILE;
+    return GNUTLS_E_FILE_ERROR;
 
   return gnutls_openpgp_add_keyring_file(&c->keyring, file);
 }
@@ -1381,25 +1357,25 @@ gnutls_certificate_set_openpgp_keyring_mem(GNUTLS_CERTIFICATE_CREDENTIALS c,
     return GNUTLS_E_INVALID_PARAMETERS;
 
   if ( !file_exist(file) )
-    return GNUTLS_E_FILE;
+    return GNUTLS_E_FILE_ERROR;
 
   if ( is_file_armored( (char*)file) )
     {
       rc = cdk_iobuf_open(&buf, file, IOBUF_MODE_RD);
       if (rc == -1)
-        return GNUTLS_E_FILE;
+        return GNUTLS_E_FILE_ERROR;
       rc = cdk_iobuf_push_filter(buf, &afx, cdk_armor_filter);
       if (rc)
         {
           cdk_iobuf_close(buf);
-          return GNUTLS_E_ASCII_ARMOR;
+          return GNUTLS_E_ASCII_ARMOR_ERROR;
         }
     }
   else
     {
       rc = cdk_iobuf_open(&buf, file, IOBUF_MODE_RD);
       if (rc == -1)
-        return GNUTLS_E_FILE;
+        return GNUTLS_E_FILE_ERROR;
     }
   data = cdk_iobuf_get_data(buf, &nbytes);
   if (data && nbytes)
@@ -1462,8 +1438,13 @@ gnutls_openpgp_recv_key(const char *host, short port, uint32 keyid,
       close(fd);
       return -1;
     }
-
+  
   request = cdk_alloc_clear(strlen(host)+100);
+  if ( request == NULL )
+    {
+      close( fd );
+      return -1;
+    }
   sprintf(request, "GET /pks/lookup?op=get&search=0x%08X HTTP/1.0\r\n"
           "Host: %s:%d\r\n", (u32)keyid, host, port);
   if ( write(fd, request, strlen(request)) == -1 )
@@ -1495,7 +1476,7 @@ gnutls_openpgp_recv_key(const char *host, short port, uint32 keyid,
   rc = cdk_iobuf_push_filter(buf, &afx, cdk_armor_filter);
   if (rc)
     {
-      rc = GNUTLS_E_ASCII_ARMOR;
+      rc = GNUTLS_E_ASCII_ARMOR_ERROR;
       goto leave;
     }
   data = cdk_iobuf_get_data(buf, &n);
@@ -1574,7 +1555,7 @@ gnutls_certificate_set_openpgp_keyserver(GNUTLS_CERTIFICATE_CREDENTIALS res,
   if (!port)
     port = 11371;
   
-  res->pgp_key_server = keyserver;
+  res->pgp_key_server = gnutls_strdup( keyserver );
   res->pgp_key_server_port = port;
 }
 
@@ -1603,7 +1584,6 @@ _gnutls_openpgp_cert2gnutls_cert(gnutls_cert *cert, gnutls_datum raw)
   return GNUTLS_E_UNIMPLEMENTED_FEATURE;  
 }
 
-
 int
 gnutls_certificate_set_openpgp_key_mem(GNUTLS_CERTIFICATE_CREDENTIALS res,
                                        gnutls_datum *cert,
@@ -1621,7 +1601,7 @@ gnutls_certificate_set_openpgp_key_file(GNUTLS_CERTIFICATE_CREDENTIALS res,
 }
 
 int
-gnutls_openpgp_extract_key_name( const gnutls_datum *cert,
+gnutls_openpgp_extract_key_name( const gnutls_datum *cert, int idx, 
                                  gnutls_openpgp_name *dn )
 {
   return GNUTLS_E_UNIMPLEMENTED_FEATURE; 
@@ -1703,6 +1683,13 @@ gnutls_certificate_set_openpgp_keyserver(GNUTLS_CERTIFICATE_CREDENTIALS res,
   return;
 }
 
+void
+gnutls_certificate_set_openpgp_trustdb(GNUTLS_CERTIFICATE_CREDENTIALS res,
+                                       char* trustdb)
+{
+  return;
+}     
+
 #endif /* HAVE_LIBOPENCDK */
 
 
