@@ -35,6 +35,7 @@
 #include "gnutls_record.h"
 #include "gnutls_datum.h"
 #include "ext_max_record.h"
+#include <gnutls_state.h>
 #include <gnutls_alert.h>
 #include <gnutls_dh.h>
 
@@ -96,150 +97,6 @@ GNUTLS_TRANSPORT_PTR gnutls_transport_get_ptr(GNUTLS_STATE state) {
 	return state->gnutls_internals.transport_ptr;
 }
 
-inline
-static void _gnutls_cal_PRF_A( MACAlgorithm algorithm, void *secret, int secret_size, void *seed, int seed_size, void* result)
-{
-	GNUTLS_MAC_HANDLE td1;
-
-	td1 = gnutls_hmac_init(algorithm, secret, secret_size);
-	gnutls_hmac(td1, seed, seed_size);
-	gnutls_hmac_deinit(td1, result);
-	
-	return;
-}
-
-#define MAX_SEED_SIZE 200
-
-/* Produces "total_bytes" bytes using the hash algorithm specified.
- * (used in the PRF function)
- */
-static int gnutls_P_hash( MACAlgorithm algorithm, opaque * secret, int secret_size, opaque * seed, int seed_size, int total_bytes, opaque* ret)
-{
-
-	GNUTLS_MAC_HANDLE td2;
-	int i = 0, times, how, blocksize, A_size;
-	opaque final[20], Atmp[MAX_SEED_SIZE];
-
-	if (seed_size > MAX_SEED_SIZE || total_bytes<=0) {
-		gnutls_assert();
-		return GNUTLS_E_INTERNAL;
-	}
-	
-	blocksize = gnutls_hmac_get_algo_len(algorithm);
-	do {
-		i += blocksize;
-	} while (i < total_bytes);
-
-	/* calculate A(0) */
-
-	memcpy( Atmp, seed, seed_size);
-	A_size = seed_size;
-
-	times = i / blocksize;
-	for (i = 0; i < times; i++) {
-		td2 = gnutls_hmac_init(algorithm, secret, secret_size);
-
-		/* here we calculate A(i+1) */
-		_gnutls_cal_PRF_A( algorithm, secret, secret_size, Atmp, A_size, Atmp);
-
-		A_size = blocksize;
-
-		gnutls_hmac(td2, Atmp, A_size);
-		gnutls_hmac(td2, seed, seed_size);
-		gnutls_hmac_deinit(td2, final);
-
-		if ( (1+i) * blocksize < total_bytes) {
-			how = blocksize;
-		} else {
-			how = total_bytes - (i) * blocksize;
-		}
-
-		if (how > 0) {
-			memcpy(&ret[i * blocksize], final, how);
-		}
-	}
-	
-	return 0;
-}
-
-/* Function that xor's buffers using the maximum word size supported
- * by the system. It should be faster.
- */
-inline static
-void _gnutls_xor(void* _o1, void* _o2, int _length) {
-unsigned long int* o1 = _o1;
-unsigned long int* o2 = _o2;
-int i, length = _length/sizeof(unsigned long int);
-int modlen = _length%sizeof(unsigned long int);
-
-	for (i = 0; i < length; i++) {
-		o1[i] ^= o2[i];
-	}
-	i*=sizeof(unsigned long int);
-	for (;i<modlen;i++) {
-		((char*)_o1)[i] ^= ((char*)_o2)[i];
-	}
-	return ;
-}
-
-#define MAX_PRF_BYTES 200
-
-/* The PRF function expands a given secret 
- * needed by the TLS specification. ret must have a least total_bytes
- * available.
- */
-int gnutls_PRF( opaque * secret, int secret_size, uint8 * label, int label_size, opaque * seed, int seed_size, int total_bytes, void* ret)
-{
-	int l_s, s_seed_size;
-	char *s1, *s2;
-	opaque s_seed[MAX_SEED_SIZE];
-	opaque o1[MAX_PRF_BYTES], o2[MAX_PRF_BYTES];
-	int result;
-
-	if (total_bytes > MAX_PRF_BYTES) {
-		gnutls_assert();
-		return GNUTLS_E_INTERNAL;
-	}
-	/* label+seed = s_seed */
-	s_seed_size = seed_size + label_size;
-
-	if (s_seed_size > MAX_SEED_SIZE) {
-		gnutls_assert();
-		return GNUTLS_E_INTERNAL;
-	}
-
-	memcpy(s_seed, label, label_size);
-	memcpy(&s_seed[label_size], seed, seed_size);
-
-	l_s = secret_size / 2;
-	s1 = &secret[0];
-	s2 = &secret[l_s];
-
-	if (secret_size % 2 != 0) {
-		l_s++;
-	}
-
-	result = gnutls_P_hash( GNUTLS_MAC_MD5, s1, l_s, s_seed, s_seed_size, total_bytes, o1);
-	if (result<0) {
-		gnutls_assert();
-		return result;
-	}
-
-	result = gnutls_P_hash( GNUTLS_MAC_SHA, s2, l_s, s_seed, s_seed_size, total_bytes, o2);
-	if (result<0) {
-		gnutls_assert();
-		return result;
-	}
-
-	_gnutls_xor(o1, o2, total_bytes);
-
-	memcpy( ret, o1, total_bytes);
-
-	return 0; /* ok */
-
-}
-
-
 /**
   * gnutls_bye - This function terminates the current TLS/SSL connection.
   * @state: is a &GNUTLS_STATE structure.
@@ -318,6 +175,62 @@ static int _gnutls_session_is_valid( GNUTLS_STATE state) {
 	return 0;
 }
 
+static
+ssize_t _gnutls_create_empty_record( GNUTLS_STATE state, ContentType type,
+	opaque** erecord)
+{
+	int cipher_size;
+	int retval;
+	int data2send;
+	uint8 headers[5];
+	GNUTLS_Version lver;
+
+	*erecord = NULL;
+
+	/* if this protection has been disabled
+	 */
+	if (state->gnutls_internals.cbc_protection_hack==0) return 0;
+
+	if (type!=GNUTLS_APPLICATION_DATA ||
+		_gnutls_cipher_is_block( gnutls_cipher_get(state))!=CIPHER_BLOCK) 
+		/* alert messages and stream ciphers
+		 * do not need this protection 
+		 */
+		return 0;
+
+	headers[0]=type;
+	
+	lver = gnutls_protocol_get_version(state);
+	if (lver==GNUTLS_VERSION_UNKNOWN) {
+		gnutls_assert();
+		return GNUTLS_E_INTERNAL_ERROR;
+	}
+
+	headers[1]=_gnutls_version_get_major( lver);
+	headers[2]=_gnutls_version_get_minor( lver);
+
+	data2send = 0;
+
+	cipher_size = _gnutls_encrypt( state, headers, RECORD_HEADER_SIZE, NULL, 0, erecord, type, 0);
+	if (cipher_size <= 0) {
+		gnutls_assert();
+		if (cipher_size==0) cipher_size = GNUTLS_E_ENCRYPTION_FAILED;
+		return cipher_size; /* error */
+	}
+
+	retval = cipher_size;
+
+	/* increase sequence number
+	 */
+	if (uint64pp( &state->connection_state.write_sequence_number) != 0) {
+		_gnutls_session_invalidate( state);
+		gnutls_assert();
+		return GNUTLS_E_RECORD_LIMIT_REACHED;
+	}
+
+	return retval;
+}
+
 /* This function behave exactly like write(). The only difference is 
  * that it accepts, the gnutls_state and the ContentType of data to
  * send (if called by the user the Content is specific)
@@ -338,6 +251,8 @@ ssize_t gnutls_send_int( GNUTLS_STATE state, ContentType type, HandshakeType hty
 	uint8 headers[5];
 	const uint8 *data=_data;
 	GNUTLS_Version lver;
+	int erecord_size = 0;
+	opaque* erecord;
 
 	if (sizeofdata == 0 || _data==NULL) {
 		gnutls_assert();
@@ -349,12 +264,14 @@ ssize_t gnutls_send_int( GNUTLS_STATE state, ContentType type, HandshakeType hty
 			return GNUTLS_E_INVALID_SESSION;
 		}
 
+
+
 	headers[0]=type;
 	
 	lver = gnutls_protocol_get_version(state);
 	if (lver==GNUTLS_VERSION_UNKNOWN) {
 		gnutls_assert();
-		return GNUTLS_E_INTERNAL;
+		return GNUTLS_E_INTERNAL_ERROR;
 	}
 
 	headers[1]=_gnutls_version_get_major( lver);
@@ -380,10 +297,24 @@ ssize_t gnutls_send_int( GNUTLS_STATE state, ContentType type, HandshakeType hty
 
 		retval = state->gnutls_internals.record_send_buffer_user_size;
 	} else {
-		cipher_size = _gnutls_encrypt( state, headers, RECORD_HEADER_SIZE, data, data2send, &cipher, type);
+
+		/* Prepend our packet with an empty record. This is to
+		 * avoid the recent CBC attacks.
+		 */
+		erecord_size = 
+			_gnutls_create_empty_record( state, type, &erecord);
+		if (erecord_size < 0) {
+			gnutls_assert();
+			return erecord_size;
+		}
+
+		/* now proceed to packet encryption
+		 */
+		cipher_size = _gnutls_encrypt( state, headers, RECORD_HEADER_SIZE, data, data2send, &cipher, type, 1);
 		if (cipher_size <= 0) {
 			gnutls_assert();
 			if (cipher_size==0) cipher_size = GNUTLS_E_ENCRYPTION_FAILED;
+			gnutls_free( erecord);
 			return cipher_size; /* error */
 		}
 
@@ -397,14 +328,16 @@ ssize_t gnutls_send_int( GNUTLS_STATE state, ContentType type, HandshakeType hty
 			gnutls_assert();
 			/* FIXME: Somebody has to do rehandshake before that.
 			 */
+			gnutls_free( erecord);
 			return GNUTLS_E_RECORD_LIMIT_REACHED;
 		}
 
-		ret = _gnutls_io_write_buffered( state, cipher, cipher_size);
+		ret = _gnutls_io_write_buffered2( state, erecord, erecord_size, cipher, cipher_size);
+		gnutls_free( erecord);
+		gnutls_free( cipher);
 	}
 
-	if ( ret != cipher_size) {
-		gnutls_free( cipher);
+	if ( ret != cipher_size + erecord_size) {
 		if ( ret < 0 && gnutls_error_is_fatal(ret)==0) {
 			/* If we have sent any data then return
 			 * that value.
@@ -426,8 +359,6 @@ ssize_t gnutls_send_int( GNUTLS_STATE state, ContentType type, HandshakeType hty
 
 	state->gnutls_internals.record_send_buffer_user_size = 0;
 
-	gnutls_free(cipher);
-
 	_gnutls_record_log( "REC: Sent Packet[%d] %s(%d) with length: %d\n",
 	(int) uint64touint32(&state->connection_state.write_sequence_number), _gnutls_packet2str(type), type, cipher_size);
 
@@ -651,6 +582,8 @@ static int _gnutls_record_check_type( GNUTLS_STATE state, ContentType recv_type,
 
 }
 
+#define MAX_EMPTY_PACKETS_SEQUENCE 4
+
 /* This function behave exactly like read(). The only difference is 
  * that it accepts, the gnutls_state and the ContentType of data to
  * send (if called by the user the Content is Userdata only)
@@ -668,7 +601,15 @@ ssize_t gnutls_recv_int( GNUTLS_STATE state, ContentType type, HandshakeType hty
 	uint8 *recv_data;
 	int ret, ret2;
 	uint16 header_size;
+	int empty_packet = 0;
 
+	begin:
+	
+	if (empty_packet > MAX_EMPTY_PACKETS_SEQUENCE) {
+		gnutls_assert();
+		return GNUTLS_E_TOO_MANY_EMPTY_PACKETS;
+	}
+	
 	/* default headers for TLS 1.0
 	 */
 	header_size = RECORD_HEADER_SIZE;
@@ -831,6 +772,13 @@ ssize_t gnutls_recv_int( GNUTLS_STATE state, ContentType type, HandshakeType hty
 		 */
 	}
 
+	/* TLS 1.0 CBC protection. Read the next fragment.
+	 */
+	if (ret==0) {
+		empty_packet++;
+		goto begin;
+	}
+	
 	return ret;
 }
 
