@@ -115,13 +115,11 @@ int i;
  * This is used to allow further handshakes.
  */
 void _gnutls_handshake_internal_state_clear( GNUTLS_STATE state) {
-	state->gnutls_internals.pgp_fingerprint = 0;
 	state->gnutls_internals.extensions_sent_size = 0;
 
 	/* by default no selected certificate */
 	state->gnutls_internals.selected_cert_index = -1;
 	state->gnutls_internals.proposed_record_size = DEFAULT_MAX_RECORD_SIZE;
-	state->gnutls_internals.send_cert_req;
 	state->gnutls_internals.adv_version_major = 0;
 	state->gnutls_internals.adv_version_minor = 0;
 	state->gnutls_internals.v2_hello = 0;
@@ -243,6 +241,11 @@ void gnutls_deinit(GNUTLS_STATE state)
 	if (state->connection_state.write_cipher_state != NULL)
 		gnutls_cipher_deinit(state->connection_state.write_cipher_state);
 
+	if (state->connection_state.read_compression_state != NULL)
+		gnutls_comp_deinit(state->connection_state.read_compression_state, 1);
+	if (state->connection_state.write_compression_state != NULL)
+		gnutls_comp_deinit(state->connection_state.write_compression_state, 0);
+
 	gnutls_sfree_datum( &state->cipher_specs.server_write_mac_secret);
 	gnutls_sfree_datum( &state->cipher_specs.client_write_mac_secret);
 	gnutls_sfree_datum( &state->cipher_specs.server_write_IV);
@@ -400,3 +403,172 @@ void _gnutls_record_set_default_version(GNUTLS_STATE state, GNUTLS_Version versi
 	state->gnutls_internals.default_record_version = version;
 }
 
+/**
+  * gnutls_record_set_cbc_protection - Used to disable the CBC protection
+  * @state: is a &GNUTLS_STATE structure.
+  * @prot: is an integer (0 or 1)
+  *
+  * A newly discovered attack against the record protocol requires some
+  * counter-measures to be taken. GnuTLS will not enable them by default
+  * thus, sends an empty record packet, before each actual record packet,
+  * in order to assure that the IV is not known to potential attackers.
+  *
+  * This function will enable or disable the chosen plaintext protection
+  * in the TLS record protocol (used with ciphers in CBC mode).
+  * if prot == 0 then protection is disabled (default), otherwise it
+  * is enabled.
+  *
+  * The protection used will slightly decrease performance, and add 
+  * 20 or more bytes per record packet.
+  *
+  **/
+void gnutls_record_set_cbc_protection(GNUTLS_STATE state, int prot)
+{
+	state->gnutls_internals.cbc_protection_hack = prot;
+}
+
+inline
+static void _gnutls_cal_PRF_A( MACAlgorithm algorithm, void *secret, int secret_size, void *seed, int seed_size, void* result)
+{
+	GNUTLS_MAC_HANDLE td1;
+
+	td1 = gnutls_hmac_init(algorithm, secret, secret_size);
+	gnutls_hmac(td1, seed, seed_size);
+	gnutls_hmac_deinit(td1, result);
+	
+	return;
+}
+
+#define MAX_SEED_SIZE 200
+
+/* Produces "total_bytes" bytes using the hash algorithm specified.
+ * (used in the PRF function)
+ */
+static int gnutls_P_hash( MACAlgorithm algorithm, opaque * secret, int secret_size, opaque * seed, int seed_size, int total_bytes, opaque* ret)
+{
+
+	GNUTLS_MAC_HANDLE td2;
+	int i = 0, times, how, blocksize, A_size;
+	opaque final[20], Atmp[MAX_SEED_SIZE];
+
+	if (seed_size > MAX_SEED_SIZE || total_bytes<=0) {
+		gnutls_assert();
+		return GNUTLS_E_INTERNAL_ERROR;
+	}
+	
+	blocksize = gnutls_hmac_get_algo_len(algorithm);
+	do {
+		i += blocksize;
+	} while (i < total_bytes);
+
+	/* calculate A(0) */
+
+	memcpy( Atmp, seed, seed_size);
+	A_size = seed_size;
+
+	times = i / blocksize;
+	for (i = 0; i < times; i++) {
+		td2 = gnutls_hmac_init(algorithm, secret, secret_size);
+
+		/* here we calculate A(i+1) */
+		_gnutls_cal_PRF_A( algorithm, secret, secret_size, Atmp, A_size, Atmp);
+
+		A_size = blocksize;
+
+		gnutls_hmac(td2, Atmp, A_size);
+		gnutls_hmac(td2, seed, seed_size);
+		gnutls_hmac_deinit(td2, final);
+
+		if ( (1+i) * blocksize < total_bytes) {
+			how = blocksize;
+		} else {
+			how = total_bytes - (i) * blocksize;
+		}
+
+		if (how > 0) {
+			memcpy(&ret[i * blocksize], final, how);
+		}
+	}
+	
+	return 0;
+}
+
+/* Function that xor's buffers using the maximum word size supported
+ * by the system. It should be faster. - only if one / and % are much faster
+ * than the whole xor operation.
+ */
+inline static
+void _gnutls_xor(void* _o1, void* _o2, int _length) {
+unsigned long int* o1 = _o1;
+unsigned long int* o2 = _o2;
+int i, length = _length/sizeof(unsigned long int);
+int modlen = _length%sizeof(unsigned long int);
+
+	for (i = 0; i < length; i++) {
+		o1[i] ^= o2[i];
+	}
+	i*=sizeof(unsigned long int);
+	for (;i<modlen;i++) {
+		((unsigned char*)_o1)[i] ^= ((unsigned char*)_o2)[i];
+	}
+	return ;
+}
+
+
+
+#define MAX_PRF_BYTES 200
+
+/* The PRF function expands a given secret 
+ * needed by the TLS specification. ret must have a least total_bytes
+ * available.
+ */
+int _gnutls_PRF( opaque * secret, int secret_size, uint8 * label, int label_size, opaque * seed, int seed_size, int total_bytes, void* ret)
+{
+	int l_s, s_seed_size;
+	char *s1, *s2;
+	opaque s_seed[MAX_SEED_SIZE];
+	opaque o1[MAX_PRF_BYTES], o2[MAX_PRF_BYTES];
+	int result;
+
+	if (total_bytes > MAX_PRF_BYTES) {
+		gnutls_assert();
+		return GNUTLS_E_INTERNAL_ERROR;
+	}
+	/* label+seed = s_seed */
+	s_seed_size = seed_size + label_size;
+
+	if (s_seed_size > MAX_SEED_SIZE) {
+		gnutls_assert();
+		return GNUTLS_E_INTERNAL_ERROR;
+	}
+
+	memcpy(s_seed, label, label_size);
+	memcpy(&s_seed[label_size], seed, seed_size);
+
+	l_s = secret_size / 2;
+	s1 = &secret[0];
+	s2 = &secret[l_s];
+
+	if (secret_size % 2 != 0) {
+		l_s++;
+	}
+
+	result = gnutls_P_hash( GNUTLS_MAC_MD5, s1, l_s, s_seed, s_seed_size, total_bytes, o1);
+	if (result<0) {
+		gnutls_assert();
+		return result;
+	}
+
+	result = gnutls_P_hash( GNUTLS_MAC_SHA, s2, l_s, s_seed, s_seed_size, total_bytes, o2);
+	if (result<0) {
+		gnutls_assert();
+		return result;
+	}
+
+	_gnutls_xor(o1, o2, total_bytes);
+
+	memcpy( ret, o1, total_bytes);
+
+	return 0; /* ok */
+
+}
