@@ -73,7 +73,7 @@ static int _IREAD(node_asn * rasn, char *name3, int name3_size, char *rstr, char
 		     asn1_create_structure(_gnutls_get_pkix(), str,
 					   &tmpasn, name2)) != ASN_OK) {
 			gnutls_assert();
-			return GNUTLS_E_ASN1_ERROR;
+			return result;
 		}
 
 		len = sizeof(str) -1;
@@ -165,7 +165,7 @@ int _gnutls_x509_get_name_type(node_asn * rasn, char *root, gnutls_DN * dn)
 			break;
 		if (result != ASN_VALUE_NOT_FOUND) {
 			gnutls_assert();
-			return GNUTLS_E_ASN1_PARSING_ERROR;
+			return result;
 		}
 
 		k2 = 0;
@@ -184,7 +184,7 @@ int _gnutls_x509_get_name_type(node_asn * rasn, char *root, gnutls_DN * dn)
 				break;
 			if (result != ASN_VALUE_NOT_FOUND) {
 				gnutls_assert();
-				return GNUTLS_E_ASN1_PARSING_ERROR;
+				return result;
 			}
 
 			_gnutls_str_cpy(name3, sizeof(name3), name2);
@@ -197,7 +197,7 @@ int _gnutls_x509_get_name_type(node_asn * rasn, char *root, gnutls_DN * dn)
 				break;
 			else if (result != ASN_OK) {
 				gnutls_assert();
-				return GNUTLS_E_ASN1_PARSING_ERROR;
+				return result;
 			}
 
 			_gnutls_str_cpy(name3, sizeof(name3), name2);
@@ -238,7 +238,7 @@ int _gnutls_x509_get_name_type(node_asn * rasn, char *root, gnutls_DN * dn)
 	if (result == ASN_ELEMENT_NOT_FOUND)
 		return 0;
 	else
-		return GNUTLS_E_ASN1_PARSING_ERROR;
+		return result;
 }
 
 
@@ -284,6 +284,10 @@ time_t _gnutls_x509_get_time(node_asn * c2, char *root, char *when)
 			ctime = _gnutls_utcTime2gtime(ttime);
 	}
 
+	/* We cannot handle dates after 2031 in 32 bit machines.
+	 * a time_t of 64bits has to be used.
+	 */
+	 	
 	if (result != ASN_OK) {
 		gnutls_assert();
 		return (time_t) (-1);
@@ -303,7 +307,7 @@ int _gnutls_x509_get_version(node_asn * c2, char *root)
 	len = sizeof(gversion) - 1;
 	if ((result = asn1_read_value(c2, name, gversion, &len)) < 0) {
 		gnutls_assert();
-		return (-1);
+		return result;
 	}
 	return (int) gversion[0] + 1;
 }
@@ -332,7 +336,7 @@ int gnutls_x509_extract_dn(const gnutls_datum * idn, gnutls_x509_dn * rdn)
 				   "PKIX1Implicit88.Name", &dn,
 				   "dn")) != ASN_OK) {
 		gnutls_assert();
-		return GNUTLS_E_ASN1_ERROR;
+		return result;
 	}
 
 	result = asn1_get_der(dn, idn->data, idn->size);
@@ -340,7 +344,7 @@ int gnutls_x509_extract_dn(const gnutls_datum * idn, gnutls_x509_dn * rdn)
 		/* couldn't decode DER */
 		gnutls_assert();
 		asn1_delete_structure(dn);
-		return GNUTLS_E_ASN1_PARSING_ERROR;
+		return result;
 	}
 
 	result = _gnutls_x509_get_name_type(dn, "dn", rdn);
@@ -379,7 +383,7 @@ int gnutls_x509_extract_certificate_dn(const gnutls_datum * cert,
 	     "certificate2")
 	    != ASN_OK) {
 		gnutls_assert();
-		return GNUTLS_E_ASN1_ERROR;
+		return result;
 	}
 
 
@@ -391,7 +395,7 @@ int gnutls_x509_extract_certificate_dn(const gnutls_datum * cert,
 
 		gnutls_assert();
 		asn1_delete_structure(c2);
-		return GNUTLS_E_ASN1_PARSING_ERROR;
+		return result;
 	}
 	if ((result =
 	     _gnutls_x509_get_name_type(c2,
@@ -431,7 +435,7 @@ int gnutls_x509_extract_certificate_issuer_dn(const gnutls_datum * cert,
 	     "certificate2")
 	    != ASN_OK) {
 		gnutls_assert();
-		return GNUTLS_E_ASN1_ERROR;
+		return result;
 	}
 
 	result = asn1_get_der(c2, cert->data, cert->size);
@@ -442,7 +446,7 @@ int gnutls_x509_extract_certificate_issuer_dn(const gnutls_datum * cert,
 
 		gnutls_assert();
 		asn1_delete_structure(c2);
-		return GNUTLS_E_ASN1_PARSING_ERROR;
+		return result;
 	}
 	if ((result =
 	     _gnutls_x509_get_name_type(c2,
@@ -510,12 +514,12 @@ int gnutls_x509_extract_subject_alt_name(const gnutls_datum * cert, int seq, cha
 		return GNUTLS_E_REQUESTED_DATA_NOT_AVAILABLE;
 	}
 
-	if (asn1_create_structure
-	    (_gnutls_get_pkix(), "PKIX1Implicit88.SubjectAltName", &c2, "san")
+	if ((result=asn1_create_structure
+	    (_gnutls_get_pkix(), "PKIX1Implicit88.SubjectAltName", &c2, "san"))
 	    != ASN_OK) {
 		gnutls_assert();
 		gnutls_free_datum( &dnsname);
-		return GNUTLS_E_ASN1_ERROR;
+		return result;
 	}
 
 	result = asn1_get_der(c2, dnsname.data, dnsname.size);
@@ -527,7 +531,7 @@ int gnutls_x509_extract_subject_alt_name(const gnutls_datum * cert, int seq, cha
 		_gnutls_log("X509_auth: Decoding error %d\n", result);
 		gnutls_assert();
 		asn1_delete_structure(c2);
-		return GNUTLS_E_ASN1_PARSING_ERROR;
+		return result;
 	}
 
 	seq++; /* 0->1, 1->2 etc */
@@ -540,7 +544,7 @@ int gnutls_x509_extract_subject_alt_name(const gnutls_datum * cert, int seq, cha
 	     asn1_read_value(c2, nptr, ext_data, &len)) != ASN_OK) {
 		gnutls_assert();
 		asn1_delete_structure(c2);
-		return GNUTLS_E_ASN1_PARSING_ERROR;
+		return result;
 	}
 
 
@@ -558,7 +562,7 @@ int gnutls_x509_extract_subject_alt_name(const gnutls_datum * cert, int seq, cha
 	     asn1_read_value(c2, nptr, ret, ret_size)) != ASN_OK) {
 		gnutls_assert();
 		asn1_delete_structure(c2);
-		return GNUTLS_E_MEMORY_ERROR;
+		return result;
 	}
 
 	asn1_delete_structure(c2);
@@ -588,7 +592,7 @@ time_t gnutls_x509_extract_certificate_activation_time(const
 	     "certificate2")
 	    != ASN_OK) {
 		gnutls_assert();
-		return -1;
+		return (time_t)-1;
 	}
 
 	result = asn1_get_der(c2, cert->data, cert->size);
@@ -598,7 +602,7 @@ time_t gnutls_x509_extract_certificate_activation_time(const
 		_gnutls_log("X509_auth: Decoding error %d\n", result);
 
 		gnutls_assert();
-		return -1;
+		return (time_t)-1;
 	}
 
 	ret = _gnutls_x509_get_time(c2, "certificate2", "notBefore");
@@ -630,7 +634,7 @@ time_t gnutls_x509_extract_certificate_expiration_time(const
 	     "certificate2")
 	    != ASN_OK) {
 		gnutls_assert();
-		return -1;
+		return (time_t)-1;
 	}
 
 	result = asn1_get_der(c2, cert->data, cert->size);
@@ -640,7 +644,7 @@ time_t gnutls_x509_extract_certificate_expiration_time(const
 		_gnutls_log("X509_auth: Decoding error %d\n", result);
 
 		gnutls_assert();
-		return -1;
+		return (time_t)-1;
 	}
 
 	ret = _gnutls_x509_get_time(c2, "certificate2", "notAfter");
@@ -663,12 +667,12 @@ int gnutls_x509_extract_certificate_version(const gnutls_datum * cert)
 	node_asn *c2;
 	int result;
 
-	if (asn1_create_structure
+	if ((result=asn1_create_structure
 	    (_gnutls_get_pkix(), "PKIX1Implicit88.Certificate", &c2,
-	     "certificate2")
+	     "certificate2"))
 	    != ASN_OK) {
 		gnutls_assert();
-		return GNUTLS_E_ASN1_ERROR;
+		return result;
 	}
 
 	result = asn1_get_der(c2, cert->data, cert->size);
@@ -678,7 +682,7 @@ int gnutls_x509_extract_certificate_version(const gnutls_datum * cert)
 		_gnutls_log("X509_auth: Decoding error %d\n", result);
 
 		gnutls_assert();
-		return GNUTLS_E_ASN1_PARSING_ERROR;
+		return result;
 	}
 
 	result = _gnutls_x509_get_version(c2, "certificate2");
@@ -712,9 +716,11 @@ int _gnutls_x509_cert_verify_peers(GNUTLS_STATE state)
 	CHECK_AUTH(GNUTLS_CRD_CERTIFICATE, GNUTLS_E_INVALID_REQUEST);
 
 	info = _gnutls_get_auth_info(state);
-	if (info == NULL)
+	if (info == NULL) {
+		gnutls_assert();
 		return GNUTLS_E_INVALID_REQUEST;
-
+	}
+	
 	cred = _gnutls_get_cred(state->gnutls_key, GNUTLS_CRD_CERTIFICATE, NULL);
 	if (cred == NULL) {
 		gnutls_assert();
@@ -890,12 +896,12 @@ int gnutls_x509_extract_certificate_serial(const gnutls_datum * cert, char* resu
 	node_asn *c2;
 	int ret;
 
-	if (asn1_create_structure
+	if ((ret=asn1_create_structure
 	    (_gnutls_get_pkix(), "PKIX1Implicit88.Certificate", &c2,
-	     "certificate2")
+	     "certificate2"))
 	    != ASN_OK) {
 		gnutls_assert();
-		return GNUTLS_E_ASN1_ERROR;
+		return ret;
 	}
 
 	ret = asn1_get_der(c2, cert->data, cert->size);
@@ -905,13 +911,13 @@ int gnutls_x509_extract_certificate_serial(const gnutls_datum * cert, char* resu
 		_gnutls_log("X509_auth: Decoding error %d\n", result);
 
 		gnutls_assert();
-		return GNUTLS_E_ASN1_PARSING_ERROR;
+		return ret;
 	}
 
 	if ((ret = asn1_read_value(c2, "certificate2.tbsCertificate.serialNumber", result, result_size)) < 0) {
 		gnutls_assert();
 		asn1_delete_structure(c2);
-		return GNUTLS_E_INVALID_REQUEST;
+		return ret;
 	}
 
 	asn1_delete_structure(c2);
@@ -1164,7 +1170,7 @@ static int read_cert_file(GNUTLS_CERTIFICATE_CREDENTIALS res, char *certfile)
 
 	fd1 = fopen(certfile, "r");
 	if (fd1 == NULL)
-		return GNUTLS_E_UNKNOWN_ERROR;
+		return GNUTLS_E_FILE_ERROR;
 
 	siz = fread(x, 1, sizeof(x), fd1);
 	fclose(fd1);
@@ -1187,7 +1193,7 @@ static int read_ca_file(GNUTLS_CERTIFICATE_CREDENTIALS res, char *cafile)
 	fd1 = fopen(cafile, "r");
 	if (fd1 == NULL) {
 		gnutls_assert();
-		return GNUTLS_E_UNKNOWN_ERROR;
+		return GNUTLS_E_FILE_ERROR;
 	}
 
 	siz = fread(x, 1, sizeof(x), fd1);
@@ -1209,7 +1215,7 @@ static int read_key_file(GNUTLS_CERTIFICATE_CREDENTIALS res, char *keyfile)
 
 	fd2 = fopen(keyfile, "r");
 	if (fd2 == NULL)
-		return GNUTLS_E_UNKNOWN_ERROR;
+		return GNUTLS_E_FILE_ERROR;
 
 	siz = fread(x, 1, sizeof(x), fd2);
 	fclose(fd2);
@@ -1404,11 +1410,11 @@ static int _read_rsa_params(opaque * der, int dersize, MPI * params)
 	int result;
 	node_asn *spk;
 
-	if (asn1_create_structure
+	if ((result=asn1_create_structure
 	    (_gnutls_get_gnutls_asn(), "GNUTLS.RSAPublicKey", &spk,
-	     "rsa_public_key") != ASN_OK) {
+	     "rsa_public_key")) != ASN_OK) {
 		gnutls_assert();
-		return GNUTLS_E_ASN1_ERROR;
+		return result;
 	}
 
 	result = asn1_get_der(spk, der, dersize);
@@ -1416,7 +1422,7 @@ static int _read_rsa_params(opaque * der, int dersize, MPI * params)
 	if (result != ASN_OK) {
 		gnutls_assert();
 		asn1_delete_structure(spk);
-		return GNUTLS_E_ASN1_PARSING_ERROR;
+		return result;
 	}
 
 
@@ -1424,7 +1430,7 @@ static int _read_rsa_params(opaque * der, int dersize, MPI * params)
 		str, sizeof(str)-1, &params[0])) < 0) {
 		gnutls_assert();
 		asn1_delete_structure(spk);
-		return GNUTLS_E_ASN1_PARSING_ERROR;
+		return GNUTLS_E_ASN1_GENERIC_ERROR;
 	}
 
 	if ( (result=_gnutls_x509_read_int( spk, "rsa_public_key.publicExponent", 
@@ -1432,7 +1438,7 @@ static int _read_rsa_params(opaque * der, int dersize, MPI * params)
 		gnutls_assert();
 		_gnutls_mpi_release(&params[0]);
 		asn1_delete_structure(spk);
-		return GNUTLS_E_ASN1_PARSING_ERROR;
+		return GNUTLS_E_ASN1_GENERIC_ERROR;
 	}
 
 	asn1_delete_structure(spk);
@@ -1452,11 +1458,11 @@ static int _read_dsa_params(opaque * der, int dersize, MPI * params)
 	int result;
 	node_asn *spk;
 
-	if (asn1_create_structure
+	if ((result=asn1_create_structure
 	    (_gnutls_get_pkix(), "PKIX1Implicit88.Dss-Parms", &spk,
-	     "dsa_parms") != ASN_OK) {
+	     "dsa_parms")) != ASN_OK) {
 		gnutls_assert();
-		return GNUTLS_E_ASN1_ERROR;
+		return result;
 	}
 
 	result = asn1_get_der(spk, der, dersize);
@@ -1464,7 +1470,7 @@ static int _read_dsa_params(opaque * der, int dersize, MPI * params)
 	if (result != ASN_OK) {
 		gnutls_assert();
 		asn1_delete_structure(spk);
-		return GNUTLS_E_ASN1_PARSING_ERROR;
+		return result;
 	}
 
 	/* FIXME: If the parameters are not included in the certificate
@@ -1476,7 +1482,7 @@ static int _read_dsa_params(opaque * der, int dersize, MPI * params)
 	if ( (result=_gnutls_x509_read_int( spk, "dsa_parms.p", str, sizeof(str)-1, &params[0])) < 0) {
 		gnutls_assert();
 		asn1_delete_structure(spk);
-		return GNUTLS_E_ASN1_PARSING_ERROR;
+		return GNUTLS_E_ASN1_GENERIC_ERROR;
 	}
 
 	/* Read q */
@@ -1485,7 +1491,7 @@ static int _read_dsa_params(opaque * der, int dersize, MPI * params)
 		gnutls_assert();
 		asn1_delete_structure(spk);
 		_gnutls_mpi_release(&params[0]);
-		return GNUTLS_E_ASN1_PARSING_ERROR;
+		return GNUTLS_E_ASN1_GENERIC_ERROR;
 	}
 
 	/* Read g */
@@ -1495,7 +1501,7 @@ static int _read_dsa_params(opaque * der, int dersize, MPI * params)
 		asn1_delete_structure(spk);
 		_gnutls_mpi_release(&params[0]);
 		_gnutls_mpi_release(&params[1]);
-		return GNUTLS_E_ASN1_PARSING_ERROR;
+		return GNUTLS_E_ASN1_GENERIC_ERROR;
 	}
 
 	asn1_delete_structure(spk);
@@ -1518,7 +1524,7 @@ static int _read_dsa_pubkey(opaque * der, int dersize, MPI * params)
 	    (_gnutls_get_gnutls_asn(), "GNUTLS.DSAPublicKey", &spk,
 	     "dsa_public_key")) != ASN_OK) {
 		gnutls_assert();
-		return GNUTLS_E_ASN1_ERROR;
+		return result;
 	}
 
 	result = asn1_get_der(spk, der, dersize);
@@ -1526,7 +1532,7 @@ static int _read_dsa_pubkey(opaque * der, int dersize, MPI * params)
 	if (result != ASN_OK) {
 		gnutls_assert();
 		asn1_delete_structure(spk);
-		return GNUTLS_E_ASN1_PARSING_ERROR;
+		return result;
 	}
 
 	/* Read p */
@@ -1534,7 +1540,7 @@ static int _read_dsa_pubkey(opaque * der, int dersize, MPI * params)
 	if ( (result=_gnutls_x509_read_int( spk, "dsa_public_key", str, sizeof(str)-1, &params[3])) < 0) {
 		gnutls_assert();
 		asn1_delete_structure(spk);
-		return GNUTLS_E_ASN1_PARSING_ERROR;
+		return GNUTLS_E_ASN1_GENERIC_ERROR;
 	}
 
 	asn1_delete_structure(spk);
@@ -1565,13 +1571,13 @@ int len, result;
 
 		if (result != ASN_OK) {
 			gnutls_assert();
-			return GNUTLS_E_ASN1_PARSING_ERROR;
+			return result;
 		}
 
 		if ((sizeof(gCert->params) / sizeof(MPI)) < RSA_PARAMS) {
 			gnutls_assert();
 			/* internal error. Increase the MPIs in params */
-			return GNUTLS_E_INTERNAL;
+			return GNUTLS_E_INTERNAL_ERROR;
 		}
 
 		if ((result =
@@ -1600,13 +1606,13 @@ int len, result;
 
 		if (result != ASN_OK) {
 			gnutls_assert();
-			return GNUTLS_E_ASN1_PARSING_ERROR;
+			return result;
 		}
 
 		if ((sizeof(gCert->params) / sizeof(MPI)) < DSA_PUBLIC_PARAMS) {
 			gnutls_assert();
 			/* internal error. Increase the MPIs in params */
-			return GNUTLS_E_INTERNAL;
+			return GNUTLS_E_INTERNAL_ERROR;
 		}
 
 		if ((result =
@@ -1625,7 +1631,7 @@ int len, result;
 
 		if (result != ASN_OK) {
 			gnutls_assert();
-			return GNUTLS_E_ASN1_PARSING_ERROR;
+			return result;
 		}
 
 		if ((result =
@@ -1678,13 +1684,13 @@ int _gnutls_x509_cert2gnutls_cert(gnutls_cert * gCert, gnutls_datum derCert)
 		return GNUTLS_E_MEMORY_ERROR;
 	}
 
-	if (asn1_create_structure
+	if ((result=asn1_create_structure
 	    (_gnutls_get_pkix(), "PKIX1Implicit88.Certificate", &c2,
-	     "certificate2")
+	     "certificate2"))
 	    != ASN_OK) {
 		gnutls_assert();
 		gnutls_free_datum( &gCert->raw);
-		return GNUTLS_E_ASN1_ERROR;
+		return result;
 	}
 
 	result = asn1_get_der(c2, derCert.data, derCert.size);
@@ -1696,7 +1702,7 @@ int _gnutls_x509_cert2gnutls_cert(gnutls_cert * gCert, gnutls_datum derCert)
 		gnutls_assert();
 		asn1_delete_structure(c2);
 		gnutls_free_datum( &gCert->raw);
-		return GNUTLS_E_ASN1_PARSING_ERROR;
+		return result;
 	}
 
 	len = sizeof(str) - 1;
@@ -1710,7 +1716,7 @@ int _gnutls_x509_cert2gnutls_cert(gnutls_cert * gCert, gnutls_datum derCert)
 		gnutls_assert();
 		asn1_delete_structure(c2);
 		gnutls_free_datum( &gCert->raw);
-		return GNUTLS_E_ASN1_PARSING_ERROR;
+		return result;
 	}
 
 	if ( (result=_gnutls_extract_x509_cert_mpi_params( str, gCert, c2, str, sizeof(str))) < 0) {
@@ -1747,7 +1753,22 @@ int _gnutls_x509_cert2gnutls_cert(gnutls_cert * gCert, gnutls_datum derCert)
 	    _gnutls_x509_get_time(c2, "certificate2", "notAfter");
 	gCert->activation_time =
 	    _gnutls_x509_get_time(c2, "certificate2", "notBefore");
+
+#if 0 
+	if (gCert->expiration_time == (time_t)(-1) ||
+		gCert->activation_time == (time_t)(-1)) {
+		gnutls_assert();
+		asn1_delete_structure(c2);
+		return GNUTLS_E_UNIX_TIME_LIMIT_EXCEEDED;
+	}
+#endif
+
 	gCert->version = _gnutls_x509_get_version(c2, "certificate2");
+	if (gCert->version < 0) {
+		gnutls_assert();
+		asn1_delete_structure(c2);
+		return GNUTLS_E_ASN1_GENERIC_ERROR;  
+	}	 
 
 	if ((result =
 	     _gnutls_get_ext_type(c2,
@@ -1809,6 +1830,7 @@ int _gnutls_check_x509_key_usage(const gnutls_cert * cert,
 	return 0;
 }
 
+#ifdef DEBUG
 
 /* Verifies a base64 encoded certificate list from memory 
  */
@@ -1890,7 +1912,7 @@ int _gnutls_verify_x509_file( char *cafile)
 	fd1 = fopen(cafile, "r");
 	if (fd1 == NULL) {
 		gnutls_assert();
-		return GNUTLS_E_UNKNOWN_ERROR;
+		return GNUTLS_E_FILE_ERROR;
 	}
 
 	siz = fread(x, 1, sizeof(x), fd1);
@@ -1901,3 +1923,4 @@ int _gnutls_verify_x509_file( char *cafile)
 	return _gnutls_verify_x509_mem( x, siz);
 }
 
+#endif
