@@ -25,12 +25,11 @@
 /*****************************************************/
 
 
-#include <gnutls_int.h> 
-
+#include <gnutls_int.h>
 #include "x509_asn1.h" 
 #include "x509_der.h"
 #include <gnutls_str.h>
-
+#include <gnutls_errors.h>
 
 /* define used for visiting trees */
 #define UP     1
@@ -436,7 +435,10 @@ _asn1_convert_integer(char *value,unsigned char *value_out,int value_out_size, i
      (!negative && (val[k]&0x80))) k--; 
 
   for(k2=k;k2<4;k2++) {
-    if (k2-k > value_out_size-1) return ASN_MEM_ERROR;
+    if (k2-k > value_out_size-1) {
+        gnutls_assert();
+    	return ASN_MEM_ERROR;
+    }
     /* VALUE_OUT is too short to contain the value convertion */
     value_out[k2-k]=val[k2];
   }
@@ -456,8 +458,9 @@ _asn1_convert_integer(char *value,unsigned char *value_out,int value_out_size, i
   *
   * Returns:
   *
-  *  ASN_OK: structure created correctly. 
-  *  ASN_GENERIC_ERROR: an error occured while structure creation  
+  *  ASN_OK\: structure created correctly. 
+  *
+  *  ASN_GENERIC_ERROR\: an error occured while structure creation  
   **/
 int
 asn1_create_tree(const static_asn *root,node_asn **pointer)
@@ -797,8 +800,9 @@ asn1_visit_tree(node_asn *pointer,char *name)
   * 
   * Returns:
   *
-  *   ASN_OK: everything OK
-  *   ASN_ELEMENT_NOT_FOUND: pointer==NULL.
+  *   ASN_OK\: everything OK
+  *
+  *   ASN_ELEMENT_NOT_FOUND\: pointer==NULL.
   *
   **/
 int
@@ -861,7 +865,7 @@ _asn1_copy_structure3(node_asn *source_node)
       if(p_s->value){
 	switch(type_field(p_s->type)){
 	case TYPE_OCTET_STRING: case TYPE_BIT_STRING: 
-	case TYPE_INTEGER: case TYPE_DEFAULT:
+	case TYPE_INTEGER:           // case TYPE_DEFAULT:
 	  len2=-1;
 	  len=_asn1_get_length_der(p_s->value,&len2);
 	  _asn1_set_value(p_d,p_s->value,len+len2);
@@ -929,8 +933,9 @@ _asn1_copy_structure2(node_asn *root,char *source_name)
   *
   * Returns:
   *
-  *  ASN_OK: creation OK
-  *  ASN_ELEMENT_NOT_FOUND: SOURCE_NAME isn't known
+  *  ASN_OK\: creation OK
+  *
+  *  ASN_ELEMENT_NOT_FOUND\: SOURCE_NAME isn't known
   * 
   * Example: using "pkix.asn"
   *  result=asn1_create_structure(cert_def,"PKIX1Implicit88.Certificate",&cert,"certificate1");
@@ -1011,9 +1016,11 @@ _asn1_append_sequence_set(node_asn *node)
   * 
   * Returns:
   * 
-  *   ASN_OK: set value OK
-  *   ASN_ELEMENT_NOT_FOUND: NAME is not a valid element.
-  *   ASN_VALUE_NOT_VALID: VALUE has a wrong format.
+  *   ASN_OK\: set value OK
+  *
+  *   ASN_ELEMENT_NOT_FOUND\: NAME is not a valid element.
+  *
+  *   ASN_VALUE_NOT_VALID\: VALUE has a wrong format.
   * 
   * Examples:  
   *   description for each type
@@ -1294,6 +1301,7 @@ asn1_write_value(node_asn *node_root,char *name,unsigned char *value,int len)
 #define PUT_VALUE( ptr, ptr_size, data, data_size) \
 	*len = data_size; \
 	if (ptr_size < data_size) { \
+		gnutls_assert(); \
 		return ASN_MEM_ERROR; \
 	} else { \
 		memcpy( ptr, data, data_size); \
@@ -1302,6 +1310,7 @@ asn1_write_value(node_asn *node_root,char *name,unsigned char *value,int len)
 #define PUT_STR_VALUE( ptr, ptr_size, data) \
 	*len = strlen(data) + 1; \
 	if (ptr_size < *len) { \
+		gnutls_assert(); \
 		return ASN_MEM_ERROR; \
 	} else { \
 		/* this strcpy is checked */ \
@@ -1311,6 +1320,7 @@ asn1_write_value(node_asn *node_root,char *name,unsigned char *value,int len)
 #define ADD_STR_VALUE( ptr, ptr_size, data) \
 	*len = strlen(data) + 1; \
 	if (ptr_size < strlen(ptr)+(*len)) { \
+		gnutls_assert(); \
 		return ASN_MEM_ERROR; \
 	} else { \
 		/* this strcat is checked */ \
@@ -1331,9 +1341,11 @@ asn1_write_value(node_asn *node_root,char *name,unsigned char *value,int len)
   * 
   * Returns:
   *
-  *   ASN_OK: set value OK
-  *   ASN_ELEMENT_NOT_FOUND: NAME is not a valid element.
-  *   ASN_VALUE_NOT_FOUND: there isn't any value for the element selected.
+  *   ASN_OK\: set value OK
+  *
+  *   ASN_ELEMENT_NOT_FOUND\: NAME is not a valid element.
+  *
+  *   ASN_VALUE_NOT_FOUND\: there isn't any value for the element selected.
   * 
   * Examples: 
   *   a description for each type
