@@ -179,13 +179,13 @@ int _gnutls_fbase64_encode(const char *msg, const uint8 * data, int data_size,
 	if (strlen(msg) > 50)
 		return -1;
 
-	strcat(top, "-----BEGIN ");
-	strcat(top, msg);
-	strcat(top, "-----");
+	strcat(top, "-----BEGIN "); /* Flawfinder: ignore */
+	strcat(top, msg); /* Flawfinder: ignore */
+	strcat(top, "-----"); /* Flawfinder: ignore */
 
-	strcat(bottom, "\n-----END ");
-	strcat(bottom, msg);
-	strcat(bottom, "-----\n");
+	strcat(bottom, "\n-----END "); /* Flawfinder: ignore */
+	strcat(bottom, msg); /* Flawfinder: ignore */
+	strcat(bottom, "-----\n"); /* Flawfinder: ignore */
 
 	ret = data_size % 3;
 	if (ret != 0)
@@ -203,7 +203,7 @@ int _gnutls_fbase64_encode(const char *msg, const uint8 * data, int data_size,
 	if ((*result) == NULL)
 		return -1;
 
-	strcat(*result, top);
+	strcat(*result, top); /* Flawfinder: ignore */
 	pos = strlen(top);
 
 	for (i = j = 0; i < data_size; i += 3, j += 4) {
@@ -239,7 +239,7 @@ int _gnutls_fbase64_encode(const char *msg, const uint8 * data, int data_size,
 		*ptr++ = tmpres[3];
 	}
 
-	strcat(*result, bottom);
+	strcat(*result, bottom); /* Flawfinder: ignore */
 	return ret;
 }
 
@@ -279,7 +279,7 @@ int size;
 /* decodes data and puts the result into result (localy alocated)
  * The result_size is the return value
  */
-int _gnutls_base64_decode(uint8 * data, int data_size, uint8 ** result)
+int _gnutls_base64_decode(const uint8 * data, int data_size, uint8 ** result)
 {
 	int i, ret, tmp, j;
 	uint8 tmpres[3];
@@ -309,7 +309,7 @@ int _gnutls_base64_decode(uint8 * data, int data_size, uint8 ** result)
 /* copies data to result but removes newlines and <CR>
  * returns the size of the data copied.
  */
-inline static int cpydata(uint8 * data, int data_size, uint8 ** result)
+inline static int cpydata(const uint8 * data, int data_size, uint8 ** result)
 {
 	int i, j;
 
@@ -326,11 +326,13 @@ inline static int cpydata(uint8 * data, int data_size, uint8 ** result)
 	return j;
 }
 
-/* decodes data and puts the result into result (localy alocated)
+/* Searches the given string for ONE PEM encoded certificate, and
+ * stores it in the result.
+ *
  * The result_size is the return value
  */
 #define ENDSTR "-----\n"
-int _gnutls_fbase64_decode( uint8 * data, int data_size,
+int _gnutls_fbase64_decode( const uint8 * data, int data_size,
 			   uint8 ** result)
 {
 	int ret;
