@@ -29,7 +29,7 @@
 #include "x509_der.h"
 #include "x509_asn1.h"
 #include <gnutls_str.h>
-
+#include <gnutls_errors.h>
 
 #define TAG_BOOLEAN          0x01
 #define TAG_INTEGER          0x02
@@ -192,9 +192,12 @@ _asn1_get_octet_der(unsigned char *der,int *der_len,unsigned char *str,int str_s
 
   if(str==NULL) return ASN_OK;
   *str_len=_asn1_get_length_der(der,&len_len);
-  if ( str_size > *str_len)
+  if ( str_size >= *str_len)
 	  memcpy(str,der+len_len,*str_len);
-  else return ASN_MEM_ERROR;
+  else {
+  	gnutls_assert();
+  	return ASN_MEM_ERROR;
+  }
   *der_len=*str_len+len_len;
   
   return ASN_OK;
@@ -369,10 +372,12 @@ _asn1_get_bit_der(unsigned char *der,int *der_len,unsigned char *str, int str_si
   if(str==NULL) return ASN_OK;
   len_byte=_asn1_get_length_der(der,&len_len)-1;
   
-  if (str_size > len_byte)
+  if (str_size >= len_byte)
  	memcpy(str,der+len_len+1,len_byte);
-  else return ASN_MEM_ERROR;
-  
+  else {
+  	gnutls_assert();
+  	return ASN_MEM_ERROR;
+  }
   *bit_len=len_byte*8-der[len_len];
   *der_len=len_byte+len_len+1;
 
@@ -826,9 +831,11 @@ _asn1_ordering_set_of(unsigned char *der,node_asn *node)
   * 
   * Returns:
   *
-  *   ASN_OK: DER encoding OK
-  *   ASN_ELEMENT_NOT_FOUND: NAME is not a valid element.
-  *   ASN_VALUE_NOT_FOUND: there is an element without a value.
+  *   ASN_OK\: DER encoding OK
+  *
+  *   ASN_ELEMENT_NOT_FOUND\: NAME is not a valid element.
+  *
+  *   ASN_VALUE_NOT_FOUND\: there is an element without a value.
   **/
 int 
 asn1_create_der(node_asn *root,char *name,unsigned char *der,int *len)
@@ -979,9 +986,11 @@ asn1_create_der(node_asn *root,char *name,unsigned char *der,int *len)
   * 
   * Returns:
   *
-  *   ASN_OK: DER encoding OK
-  *   ASN_ELEMENT_NOT_FOUND: NAME is not a valid element.
-  *   ASN_TAG_ERROR,ASN_DER_ERROR: the der encoding doesn't match the structure NAME.  
+  *   ASN_OK\: DER encoding OK
+  *
+  *   ASN_ELEMENT_NOT_FOUND\: NAME is not a valid element.
+  *
+  *   ASN_TAG_ERROR,ASN_DER_ERROR\: the der encoding doesn't match the structure NAME.  
   **/
 
 int 
@@ -1220,9 +1229,11 @@ asn1_get_der(node_asn *root,unsigned char *der,int len)
   *
   * Returns:
   *
-  *   ASN_OK: DER encoding OK
-  *   ASN_ELEMENT_NOT_FOUND: NAME or NAME_ELEMENT is not a valid element.
-  *   ASN_TAG_ERROR,ASN_DER_ERROR: the der encoding doesn't match the structure NAME.
+  *   ASN_OK\: DER encoding OK
+  *
+  *   ASN_ELEMENT_NOT_FOUND\: NAME or NAME_ELEMENT is not a valid element.
+  *
+  *   ASN_TAG_ERROR,ASN_DER_ERROR\: the der encoding doesn't match the structure NAME.
   *
   **/
 int 
