@@ -103,7 +103,7 @@ void print_openpgp_info(GNUTLS_STATE state)
 			       printable);
 
 			gnutls_openpgp_extract_key_name(&cert_list
-							      [0], &pgp_name);
+							      [0], 0, &pgp_name);
 			PRINT_PGP_NAME(pgp_name);
 
 		}
@@ -132,10 +132,10 @@ void print_cert_vrfy(GNUTLS_STATE state)
 	if (status & GNUTLS_CERT_EXPIRED)
 		printf
 		    ("- Peer's certificate is expired\n");
-	if (status & GNUTLS_CERT_TRUSTED)
-		printf("- Peer's certificate is trusted\n");
 	if (status & GNUTLS_CERT_NOT_TRUSTED)
 		printf("- Peer's certificate is NOT trusted\n");
+	else
+		printf("- Peer's certificate is trusted\n");
 	if (status & GNUTLS_CERT_CORRUPTED)
 		printf("- Peer's certificate is corrupted\n");
 
